@@ -1,7 +1,7 @@
 use super::*;
 use crate::{
     core::{constants::MAX_CREW_SIZE, utils::is_default},
-    game_engine::{tactic::Tactic, types::EnginePlayer, Tournament, TournamentId},
+    game_engine::{tactic::Tactic, types::EnginePlayer, Tournament, TournamentId, TournamentState},
     network::{challenge::Challenge, trade::Trade},
     types::*,
 };
@@ -26,13 +26,16 @@ pub struct CrewRoles {
     pub mozzo: Vec<PlayerId>,
 }
 
-#[derive(Debug, Default, Display, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(Debug, Default, Display, Serialize, Deserialize, Clone, Copy, PartialEq)]
 pub enum TournamentRegistrationState {
     #[default]
     None,
     Pending {
         tournament_id: TournamentId,
     },
+    Registered {
+        tournament_id: TournamentId,
+    },
     Confirmed {
         tournament_id: TournamentId,
     },
@@ -72,6 +75,9 @@ pub struct Team {
     pub tournament_registration_state: TournamentRegistrationState,
     #[serde(skip_serializing_if = "is_default")]
     #[serde(default)]
+    pub is_organizing_tournament: Option<TournamentId>,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
     pub local_game_rating: GameRating,
     #[serde(skip_serializing_if = "is_default")]
     #[serde(default)]
@@ -100,7 +106,10 @@ pub struct Team {
     pub honours: HashSet<Honour>,
     #[serde(skip_serializing_if = "is_default")]
     #[serde(default)]
-    pub space_cove: SpaceCoveState,
+    pub space_cove: Option<SpaceCove>,
+    #[serde(skip_serializing_if = "is_default")]
+    #[serde(default)]
+    pub tournaments_won: Vec<TournamentId>,
 }
 
 impl Team {
@@ -271,10 +280,20 @@ impl Team {
         }
     }
 
+    pub fn playing_in_tournament(&self) -> Option<PlanetId> {
+        match self.tournament_registration_state {
+            TournamentRegistrationState::None
+            | TournamentRegistrationState::Pending { .. }
+            | TournamentRegistrationState::Registered { .. } => None,
+            TournamentRegistrationState::Confirmed { tournament_id } => Some(tournament_id),
+        }
+    }
+
     pub fn committed_to_tournament(&self) -> Option<PlanetId> {
         match self.tournament_registration_state {
             TournamentRegistrationState::None => None,
             TournamentRegistrationState::Pending { tournament_id }
+            | TournamentRegistrationState::Registered { tournament_id }
             | TournamentRegistrationState::Confirmed { tournament_id } => Some(tournament_id),
         }
     }
@@ -285,7 +304,7 @@ impl Team {
             .iter()
             .take(MAX_PLAYERS_PER_GAME)
             .map(|&id| {
-                if let Ok(player) = world.get_player_or_err(&id) {
+                if let Ok(player) = world.players.get_or_err(&id) {
                     player.current_tiredness(world)
                 } else {
                     0.0
@@ -302,12 +321,7 @@ impl Team {
     }
 
     pub fn has_space_cove_on(&self) -> Option<PlanetId> {
-        match self.space_cove {
-            SpaceCoveState::None => None,
-            SpaceCoveState::Pending { planet_id } | SpaceCoveState::Ready { planet_id } => {
-                Some(planet_id)
-            }
-        }
+        self.space_cove.as_ref().map(|cove| cove.planet_id)
     }
 
     pub fn can_teleport_to(&self, to: &Planet) -> AppResult<()> {
@@ -407,7 +421,7 @@ impl Team {
             return Err(anyhow!("{} is playing", self.name));
         }
 
-        if self.committed_to_tournament().is_some() {
+        if self.playing_in_tournament().is_some() {
             return Err(anyhow!("{} is in a tournament", self.name));
         }
 
@@ -423,20 +437,26 @@ impl Team {
             return Err(anyhow!("{} is playing", self.name));
         }
 
-        if self.committed_to_tournament().is_some() {
+        if self.playing_in_tournament().is_some() {
             return Err(anyhow!("{} is in a tournament", self.name));
         }
 
         Ok(())
     }
 
-    fn can_play_game_with_team(&self, team: &Team) -> AppResult<()> {
-        if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
+    fn can_play_game_with_team(
+        &self,
+        team: &Team,
+        part_of_tournament: Option<TournamentId>,
+    ) -> AppResult<()> {
+        if self.playing_in_tournament() != part_of_tournament {
+            return Err(anyhow!(
+                "{} game and team tournaments not matching",
+                self.name
+            ));
         }
-
-        if team.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", team.name));
+        if team.playing_in_tournament() != part_of_tournament {
+            return Err(anyhow!("Team game and team tournaments not matching"));
         }
 
         if self.id == team.id {
@@ -448,7 +468,13 @@ impl Team {
         }
 
         if self.is_on_planet() != team.is_on_planet() {
-            return Err(anyhow!("Not on the same planet"));
+            return Err(anyhow!(
+                "{} and {} not on the same planet: {:#?} and {:#?}",
+                self.name,
+                team.name,
+                self.is_on_planet(),
+                team.is_on_planet()
+            ));
         }
 
         if self.player_ids.len() < MIN_PLAYERS_PER_GAME {
@@ -468,23 +494,33 @@ impl Team {
         }
 
         if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
+            return Err(anyhow!("{} is already commited to a tournament", self.name));
         }
 
-        match self.space_cove {
-            SpaceCoveState::Ready { planet_id } => {
-                if !matches!(self.is_on_planet(), Some(id) if id == planet_id) {
-                    return Err(anyhow!(
-                        "Cannot organize a tournament while not at your space cove planet."
-                    ));
-                }
-            }
+        if self.is_organizing_tournament.is_some() {
+            return Err(anyhow!("{} is already organizing a tournament", self.name));
+        }
 
-            _ => {
+        match self.space_cove.as_ref() {
+            None => {
                 return Err(anyhow!(
                     "Cannot organize a tournament without a space cove."
                 ));
             }
+            Some(cove) => match cove.state {
+                SpaceCoveState::UnderConstruction => {
+                    return Err(anyhow!(
+                        "Cannot organize a tournament if space cove is not ready."
+                    ))
+                }
+                SpaceCoveState::Ready => {
+                    if !matches!(self.is_on_planet(), Some(id) if id == cove.planet_id) {
+                        return Err(anyhow!(
+                            "Cannot organize a tournament while not at your space cove planet."
+                        ));
+                    }
+                }
+            },
         }
 
         // FIXME: add conditions on kartoffeln
@@ -495,10 +531,60 @@ impl Team {
     pub fn can_register_to_tournament(
         &self,
         tournament: &Tournament,
-        team_rating: Skill,
+        timestamp: Tick,
     ) -> AppResult<()> {
-        if tournament.has_started(Tick::now()) {
-            return Err(anyhow!("Tournament has already started."));
+        if !matches!(tournament.state(timestamp), TournamentState::Registration) {
+            return Err(anyhow!("Tournament registrations are closed."));
+        }
+
+        if tournament.is_team_registered(&self.id) {
+            return Err(anyhow!("Team is already registered to this tournament."));
+        }
+
+        if self.current_game.is_some() {
+            return Err(anyhow!("Team is playing a game."));
+        }
+
+        // Only allowed state are None and Pending
+        if matches!(
+            self.tournament_registration_state,
+            TournamentRegistrationState::Registered { tournament_id } if tournament_id != tournament.id
+        ) {
+            return Err(anyhow!("Team is registered to another tournament."));
+        }
+
+        if matches!(
+            self.tournament_registration_state,
+            TournamentRegistrationState::Registered { tournament_id } if tournament_id == tournament.id
+        ) {
+            return Err(anyhow!("Team is registered to this tournament."));
+        }
+
+        if matches!(
+            self.tournament_registration_state,
+            TournamentRegistrationState::Confirmed { .. }
+        ) {
+            return Err(anyhow!("Team is playing in a tournament."));
+        }
+
+        if !matches!(self.is_on_planet(), Some(id) if id == tournament.planet_id) {
+            return Err(anyhow!("Team is not at the tournament location."));
+        }
+
+        Ok(())
+    }
+
+    pub fn can_confirm_tournament_registration(
+        &self,
+        tournament: &Tournament,
+        timestamp: Tick,
+    ) -> AppResult<()> {
+        if !matches!(tournament.state(timestamp), TournamentState::Confirmation) {
+            return Err(anyhow!("Tournament confirmations are closed."));
+        }
+
+        if !tournament.is_team_registered(&self.id) {
+            return Err(anyhow!("Team is not registered to this tournament."));
         }
 
         if tournament.participants.len() == tournament.max_participants {
@@ -509,16 +595,20 @@ impl Team {
             return Err(anyhow!("Team is playing a game."));
         }
 
-        if !matches!(self.is_on_planet(), Some(id) if id == tournament.planet_id) {
-            return Err(anyhow!("Team is not at the tournament location."));
+        if self.id != tournament.organizer_id
+            && !matches!(
+                self.tournament_registration_state,
+                TournamentRegistrationState::Registered { tournament_id } if tournament_id == tournament.id
+            )
+        {
+            return Err(anyhow!(
+                "Team {} is not Registered for this tournament.",
+                self.name
+            ));
         }
 
-        if team_rating < tournament.minimum_team_rating {
-            return Err(anyhow!(
-                "Team rating is too low ({} < {}).",
-                team_rating,
-                tournament.minimum_team_rating
-            ));
+        if !matches!(self.is_on_planet(), Some(id) if id == tournament.planet_id) {
+            return Err(anyhow!("Team is not at the tournament location."));
         }
 
         Ok(())
@@ -529,7 +619,7 @@ impl Team {
         // but crucially skips the checks about the current_game.
         // This is to go around a race condition described in the challenge SynAck protocol.
 
-        self.can_play_game_with_team(team)
+        self.can_play_game_with_team(team, None)
     }
 
     pub fn can_challenge_local_team(&self, team: &Team) -> AppResult<()> {
@@ -545,7 +635,7 @@ impl Team {
             return Err(anyhow!("{} is already playing", team.name));
         }
 
-        self.can_play_game_with_team(team)
+        self.can_play_game_with_team(team, None)
     }
 
     pub fn can_challenge_network_team(&self, team: &Team) -> AppResult<()> {
@@ -565,7 +655,7 @@ impl Team {
             return Err(anyhow!("Already challenged {}", team.name));
         }
 
-        self.can_play_game_with_team(team)
+        self.can_play_game_with_team(team, None)
     }
 
     pub fn can_trade_players(
@@ -603,12 +693,12 @@ impl Team {
             return Err(anyhow!("{} is playing", target_team.name));
         }
 
-        if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
+        if self.playing_in_tournament().is_some() {
+            return Err(anyhow!("{} is playing in a tournament", self.name));
         }
 
-        if target_team.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", target_team.name));
+        if target_team.playing_in_tournament().is_some() {
+            return Err(anyhow!("{} is playing in a tournament", target_team.name));
         }
 
         Ok(())
@@ -637,8 +727,8 @@ impl Team {
             return Err(anyhow!("{} is playing", self.name));
         }
 
-        if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
+        if self.playing_in_tournament().is_some() {
+            return Err(anyhow!("{} is playing in a tournament", self.name));
         }
 
         let is_teleporting = duration == TELEPORT_TRAVEL_DURATION;
@@ -687,8 +777,8 @@ impl Team {
             return Err(anyhow!("{} is playing", self.name));
         }
 
-        if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
+        if self.playing_in_tournament().is_some() {
+            return Err(anyhow!("{} is playing in a tournament", self.name));
         }
 
         if self.spaceship.current_durability() == 0 {
@@ -738,10 +828,6 @@ impl Team {
             return Err(anyhow!("{} is playing", self.name));
         }
 
-        if self.committed_to_tournament().is_some() {
-            return Err(anyhow!("{} is in a tournament", self.name));
-        }
-
         Ok(())
     }
 
@@ -808,8 +894,7 @@ impl Team {
         }
 
         // Special rules for space cove: it has to be unique across all asteroids.
-        if upgrade.target == AsteroidUpgradeTarget::SpaceCove && self.has_space_cove_on().is_some()
-        {
+        if upgrade.target == AsteroidUpgradeTarget::SpaceCove && self.space_cove.is_some() {
             return Err(anyhow!("You already have a space cove"));
         }
 
@@ -840,7 +925,7 @@ impl Team {
         }
 
         if self.is_on_planet() != Some(asteroid.id) {
-            return Err(anyhow!("Can only upgrade when the team is on the asteroid"));
+            return Err(anyhow!("Can only build on the asteroid"));
         }
 
         for (resource, amount) in upgrade.target.upgrade_cost().iter() {
