@@ -3,12 +3,14 @@ use super::trade::Trade;
 use super::types::{NetworkData, NetworkGame, NetworkRequestState, NetworkTeam, SeedInfo};
 use crate::app_version;
 use crate::core::constants::NETWORK_GAME_START_DELAY;
-use crate::core::{TournamentRegistrationState, MAX_AVG_TIREDNESS_PER_AUTO_GAME};
+use crate::core::{Team, TournamentRegistrationState, World, MAX_AVG_TIREDNESS_PER_AUTO_GAME};
+use crate::game_engine::game::GameSummary;
 use crate::game_engine::types::TeamInGame;
-use crate::game_engine::{Tournament, TournamentId};
+use crate::game_engine::{Tournament, TournamentId, TournamentState};
+use crate::network::types::TournamentRequestState;
 use crate::store::deserialize;
-use crate::types::{AppResult, PlayerMap, SystemTimeTick, TeamId, Tick};
-use crate::ui::PopupMessage;
+use crate::types::{AppResult, HashMapWithResult, PlayerMap, SystemTimeTick, TeamId, Tick};
+use crate::ui::{PopupMessage, UiScreen};
 use crate::{app::App, types::AppCallback};
 use anyhow::anyhow;
 use libp2p::gossipsub::TopicHash;
@@ -178,8 +180,61 @@ impl NetworkCallback {
     fn handle_tournament_registration_request_topic(
         tournament_id: TournamentId,
         team_id: TeamId,
-        state: TournamentRegistrationState,
+        team_data: Option<(Team, PlayerMap)>,
+        request_state: TournamentRequestState,
     ) -> AppCallback {
+        // ---------------------------------------- creation time -------------------------------------------------
+        //
+        //                      ------------                             -------------
+        //     set Pending      |          |  - RegistrationRequest ->   |           | can_register_to_tournament?
+        //                      |          |                             |           |
+        //                      |          |                             |           |
+        //  set Registered      |   team   |  <--- RegistrationOk ---    | organizer |  is_ok()
+        //                      |          |                             |           |
+        //     set None         |          |  <---- RegDeclined ----     |           |  is_err()
+        //                      |          |                             |           |
+        //                      |          |                             |           |
+        //                      ------------                             -------------
+        //
+        // ---------------------------------- registration closed time --------------------------------------------
+        //
+        //                      ------------                             -------------
+        //   can_register_..?   |          |  <-- ConfirmationRequest -- |           |
+        //     && Registered    |          |                             |           |
+        //                      |          |                             |           |
+        //        is_ok()       |   team   |  - ParticipationRequest ->  | organizer | --- can_participate_?-|
+        //                      |          |                             |           |                       |
+        //       is_err()       |          |  ---- ConfDeclined --->     |           |                       |
+        //                      |          |                             |           |                       |
+        //      set Confirmed   |          |  <--- ParticipationOk --    |           | -------is_ok()------ /|
+        //      set None.       |          |  <--- PartDeclined -----    |           | -------is_err()------/
+        //                      ------------                             -------------
+        //
+        // ---------------------------------- Confirmation time -------------------------------------------------
+        //
+        //                      ------------                             -------------
+        //                      |          |                             |           |
+        //                      |   team   |  <----- Tournament ----     | organizer |
+        //                      |          |                             |           |
+        //                      ------------                             -------------
+
+        // ---------------------------------- Syncing time ------------------------------------------------------
+        // ---------------------------------- Tournarment start time --------------------------------------------
+
+        fn discard_request_log_event(
+            ui: &mut UiScreen,
+            request_state: &TournamentRequestState,
+            tournament_state: &TournamentState,
+        ) {
+            ui.push_log_event(
+                Tick::now(),
+                None,
+                format!(
+                    "Discard tournament request with state {request_state} for tournament state {tournament_state}"
+                ),
+                log::Level::Error,
+            );
+        }
         Box::new(move |app: &mut App| {
             let tournament = if let Some(t) = app.world.tournaments.get_mut(&tournament_id) {
                 t
@@ -187,125 +242,326 @@ impl NetworkCallback {
                 return Err(anyhow!("No tournament with id {tournament_id} found."));
             };
 
-            // Check that state and tournament_id are consistent
-            match state {
-                TournamentRegistrationState::Pending {
-                    tournament_id: state_tid,
-                }
-                | TournamentRegistrationState::Confirmed {
-                    tournament_id: state_tid,
-                } => {
-                    if tournament_id != state_tid {
-                        return Err(anyhow!(
-                            "Inconsistent tournament ids: {tournament_id} and {state_tid}"
-                        ));
-                    }
-                }
-                TournamentRegistrationState::None => {}
-            }
-
             // Check if own_team is tournament organizer.
-            // In this case, we register the team to the tournament and send back the response.
             if tournament.organizer_id == app.world.own_team_id {
-                // We process only requests with state = TournamentRegistrationState::Pending { tournament_id }
-                if !matches!(state, TournamentRegistrationState::Pending { .. }) {
-                    app.network_handler.send_tournament_registration_request(
-                        tournament_id,
-                        team_id,
-                        TournamentRegistrationState::None,
-                    )?;
-                    app.ui.push_log_event(
+                assert!(team_id != app.world.own_team_id);
+                // Only possible combinations are
+                // TournamentState::Registration && state = TournamentRequestState::RegistrationRequest
+                // TournamentState::Confirmation  && state = TournamentRequestState::ParticipationRequest|Declined
+                let timestamp = Tick::now();
+                match tournament.state(timestamp) {
+                    TournamentState::Registration => {
+                        // We process only requests with state = TournamentRequestState::RegistrationRequest
+                        if request_state != TournamentRequestState::RegistrationRequest {
+                            let reason =
+                                format!("Invalid tournament request received: {request_state}");
+                            app.network_handler.send_tournament_registration_request(
+                                tournament_id,
+                                team_id,
+                                None,
+                                TournamentRequestState::RegistrationDeclined {
+                                    reason: reason.clone(),
+                                },
+                            )?;
+                            app.ui.push_log_event(
                         Tick::now(),
-                        None,
-                        format!(
-                            "Registration declined by organizer: Invalid tournament state received: {state}"
-                        ),
-                        log::Level::Error,
-                    );
-                }
+                            None,
+                            format!(
+                                "Registration to tournament {tournament_id} declined for team {team_id}: {reason}."
+                            ),
+                            log::Level::Error,
+                        );
+                        }
 
-                let team = app
-                    .world
-                    .teams
-                    .get(&team_id)
-                    .ok_or(anyhow!("Team {team_id} not found."))?;
-                let mut players = PlayerMap::new();
-                for player_id in team.player_ids.iter() {
-                    let player = app
-                        .world
-                        .players
-                        .get(player_id)
-                        .ok_or(anyhow!("Player {player_id} not found."))?
-                        .clone();
-                    players.insert(player.id, player);
-                }
-                match tournament.register_team(team, players) {
-                    Ok(()) => {
-                        app.network_handler.send_tournament_registration_request(
-                            tournament_id,
-                            team_id,
-                            TournamentRegistrationState::Confirmed { tournament_id },
-                        )?;
-                        app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} confirmed for team {team_id}."), log::Level::Info);
-                        app.network_handler.send_tournament(tournament.clone())?;
+                        let (team, players) = if let Some(data) = team_data.as_ref() {
+                            data
+                        } else {
+                            let reason = "Missing team data".to_string();
+                            app.network_handler.send_tournament_registration_request(
+                                tournament_id,
+                                team_id,
+                                None,
+                                TournamentRequestState::RegistrationDeclined {
+                                    reason: reason.clone(),
+                                },
+                            )?;
+                            app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {reason}."), log::Level::Warn);
+                            return Ok(None);
+                        };
+
+                        match tournament.register_team(team, players.clone(), Tick::now()) {
+                            Ok(()) => {
+                                app.network_handler.send_tournament_registration_request(
+                                    tournament_id,
+                                    team_id,
+                                    None,
+                                    TournamentRequestState::RegistrationOk,
+                                )?;
+                                app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} succesful for team {team_id}."), log::Level::Info);
+                                app.network_handler.send_tournament(tournament.clone())?;
+                            }
+
+                            Err(err) => {
+                                app.network_handler.send_tournament_registration_request(
+                                    tournament_id,
+                                    team_id,
+                                    None,
+                                    TournamentRequestState::RegistrationDeclined {
+                                        reason: err.to_string(),
+                                    },
+                                )?;
+                                app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {err}."), log::Level::Warn);
+                            }
+                        }
                     }
-                    Err(err) => {
-                        app.network_handler.send_tournament_registration_request(
-                            tournament_id,
-                            team_id,
-                            TournamentRegistrationState::None,
-                        )?;
-                        app.ui.push_log_event(Tick::now(), None, format!("Registration to tournament {tournament_id} declined for team {team_id}: {err}"), log::Level::Warn);
+
+                    TournamentState::Confirmation => {
+                        // We process only requests with
+                        // TournamentRequestState::ParticipationRequest|Declined
+                        match &request_state {
+                            TournamentRequestState::ConfirmationDeclined { reason } => {
+                                // Team did not confirm registration
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    format!(
+                                        "Team {team_id} did not confirm registration to tournament: {reason}."
+                                    ),
+                                    log::Level::Warn,
+                                );
+                            }
+                            TournamentRequestState::ParticipationRequest => {
+                                let (team, players) = if let Some(data) = team_data.as_ref() {
+                                    data
+                                } else {
+                                    let reason = "missing team data".to_string();
+                                    app.network_handler.send_tournament_registration_request(
+                                        tournament_id,
+                                        team_id,
+                                        None,
+                                        TournamentRequestState::ParticipationDeclined {
+                                            reason: reason.clone(),
+                                        },
+                                    )?;
+                                    app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} declined for team {team_id}: {reason}."), log::Level::Warn);
+                                    return Ok(None);
+                                };
+                                match tournament.confirm_team_registration(
+                                    team,
+                                    players.clone(),
+                                    Tick::now(),
+                                ) {
+                                    Ok(()) => {
+                                        app.network_handler.send_tournament_registration_request(
+                                            tournament_id,
+                                            team_id,
+                                            None,
+                                            TournamentRequestState::ParticipationOk,
+                                        )?;
+                                        app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} confirmed for team {team_id}."), log::Level::Info);
+                                    }
+                                    Err(err) => {
+                                        app.network_handler.send_tournament_registration_request(
+                                            tournament_id,
+                                            team_id,
+                                            None,
+                                            TournamentRequestState::ParticipationDeclined {
+                                                reason: err.to_string(),
+                                            },
+                                        )?;
+                                        app.ui.push_log_event(Tick::now(), None, format!("Participation to tournament {tournament_id} declined for team {team_id}: {err}."), log::Level::Warn);
+                                    }
+                                }
+                            }
+                            _ => discard_request_log_event(
+                                &mut app.ui,
+                                &request_state,
+                                &tournament.state(timestamp),
+                            ),
+                        }
                     }
+
+                    _ => discard_request_log_event(
+                        &mut app.ui,
+                        &request_state,
+                        &tournament.state(timestamp),
+                    ),
                 }
             }
             // Check if own_team is team_id (sender of request).
-            // In this case we intepret the state as a confirmation or a cancelation,
-            // if the state of the team is pending and the tournament id matches.
             else if team_id == app.world.own_team_id {
-                let own_team = app.world.get_own_team_mut()?;
-                if !matches!(
-                    own_team.tournament_registration_state,
-                    TournamentRegistrationState::Pending {tournament_id: id} if id == tournament_id
-                ) {
-                    // Discard message
-                    app.ui.push_log_event(
-                        Tick::now(),
-                        None,
-                        format!("Invalid tournament state received: {state}"),
-                        log::Level::Error,
-                    );
-                }
+                // Only possible combinations are
+                // TournamentState::Registration && state = TournamentRegistrationState::(Registered || None)
+                // TournamentState::Confirmation  && state = TournamentRegistrationState::(Pending || Confirmed || None)
+                let own_team = app
+                    .world
+                    .teams
+                    .get_mut(&app.world.own_team_id)
+                    .expect("Own team should exist");
 
-                // We process only requests with state = TournamentRegistrationState::Confirmed { tournament_id } or None
-                match state {
-                    TournamentRegistrationState::None => {
-                        app.ui.push_log_event(
-                            Tick::now(),
-                            None,
-                            format!("Registration to tournament {tournament_id} declined."),
-                            log::Level::Warn,
-                        );
-                        own_team.tournament_registration_state = TournamentRegistrationState::None;
+                let timestamp = Tick::now();
+
+                match tournament.state(timestamp) {
+                    TournamentState::Registration => {
+                        if !matches!(
+                            own_team.tournament_registration_state,
+                            TournamentRegistrationState::Pending {tournament_id: id} if id == tournament_id
+                        ) {
+                            discard_request_log_event(
+                                &mut app.ui,
+                                &request_state,
+                                &tournament.state(timestamp),
+                            );
+                            return Ok(None);
+                        }
+
+                        // We process only requests with state = TournamentRequestState::RegistrationOk|Declined
+                        match &request_state {
+                            TournamentRequestState::RegistrationDeclined { reason } => {
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    format!("Registration to tournament {tournament_id} declined: {reason}."),
+                                    log::Level::Warn,
+                                );
+                                own_team.tournament_registration_state =
+                                    TournamentRegistrationState::None;
+                            }
+
+                            TournamentRequestState::RegistrationOk => {
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    format!("Registration to tournament {tournament_id} accepted."),
+                                    log::Level::Info,
+                                );
+                                own_team.tournament_registration_state =
+                                    TournamentRegistrationState::Registered { tournament_id };
+                            }
+
+                            _ => discard_request_log_event(
+                                &mut app.ui,
+                                &request_state,
+                                &tournament.state(timestamp),
+                            ),
+                        }
                     }
-                    TournamentRegistrationState::Pending { tournament_id } => {
-                        app.ui.push_log_event(
-                            Tick::now(),
-                            None,
-                            format!("Invalid tournament {tournament_id} state received: {state}"),
-                            log::Level::Error,
-                        );
-                    }
-                    TournamentRegistrationState::Confirmed { tournament_id } => {
-                        app.ui.push_log_event(
-                            Tick::now(),
-                            None,
-                            format!("Registration to tournament {tournament_id} accepted."),
-                            log::Level::Info,
-                        );
-                        own_team.tournament_registration_state =
-                            TournamentRegistrationState::Confirmed { tournament_id };
+
+                    TournamentState::Confirmation => {
+                        //If we are already Confirmed, return.
+                        if matches!(
+                            own_team.tournament_registration_state,
+                            TournamentRegistrationState::Confirmed {tournament_id: id} if id == tournament_id
+                        ) {
+                            app.ui.push_log_event(
+                                Tick::now(),
+                                None,
+                                "Tournament participation already confirmed.".to_string(),
+                                log::Level::Warn,
+                            );
+                            return Ok(None);
+                        }
+
+                        // Basically if tournament_registration_state == None | Pending
+                        if !matches!(
+                            own_team.tournament_registration_state,
+                            TournamentRegistrationState::Registered {tournament_id: id} if id == tournament_id
+                        ) {
+                            own_team.tournament_registration_state =
+                                TournamentRegistrationState::None;
+
+                            discard_request_log_event(
+                                &mut app.ui,
+                                &request_state,
+                                &tournament.state(timestamp),
+                            );
+                            return Ok(None);
+                        }
+
+                        // Here we know that tournament_registration_state == Registered
+                        // We process only requests with TournamentRequestState::(ConfirmationRequest || ParticipationOk|Declined)
+                        match &request_state {
+                            // We are asked to confirm our participation.
+                            TournamentRequestState::ConfirmationRequest => {
+                                // First, check if we can participate.
+                                if let Err(err) = own_team
+                                    .can_confirm_tournament_registration(tournament, Tick::now())
+                                {
+                                    app.network_handler.send_tournament_registration_request(
+                                        tournament_id,
+                                        own_team.id,
+                                        None,
+                                        TournamentRequestState::ConfirmationDeclined {
+                                            reason: err.to_string(),
+                                        },
+                                    )?;
+                                    own_team.tournament_registration_state =
+                                        TournamentRegistrationState::None;
+                                    app.ui.push_log_event(
+                                        Tick::now(),
+                                        None,
+                                        format!("Cannot confirm tournament participation: {err}."),
+                                        log::Level::Warn,
+                                    );
+                                    return Ok(None);
+                                }
+
+                                // We can participate, so we answer with a ParticipationRequest.
+                                let mut team = own_team.clone();
+                                team.peer_id = Some(*app.network_handler.own_peer_id());
+                                let players =
+                                    World::get_game_players_by_team(&app.world.players, &team)?;
+                                let team_data = Some((team, players));
+                                app.network_handler.send_tournament_registration_request(
+                                    tournament_id,
+                                    own_team.id,
+                                    team_data,
+                                    TournamentRequestState::ParticipationRequest,
+                                )?;
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    "Tournament participation request sent, waiting for response."
+                                        .to_string(),
+                                    log::Level::Info,
+                                )
+                            }
+
+                            // Ourparticipation was declined :(
+                            TournamentRequestState::ParticipationDeclined { reason } => {
+                                own_team.tournament_registration_state =
+                                    TournamentRegistrationState::None;
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    format!("Tournament participation declined: {reason}."),
+                                    log::Level::Warn,
+                                );
+                            }
+                            TournamentRequestState::ParticipationOk => {
+                                // Our participation was confirmed!
+                                own_team.tournament_registration_state =
+                                    TournamentRegistrationState::Confirmed { tournament_id };
+                                app.ui.push_log_event(
+                                    Tick::now(),
+                                    None,
+                                    "Tournament participation confirmed!".to_string(),
+                                    log::Level::Info,
+                                )
+                            }
+                            _ => discard_request_log_event(
+                                &mut app.ui,
+                                &request_state,
+                                &tournament.state(timestamp),
+                            ),
+                        }
                     }
+
+                    _ => discard_request_log_event(
+                        &mut app.ui,
+                        &request_state,
+                        &tournament.state(timestamp),
+                    ),
                 }
             }
 
@@ -321,25 +577,55 @@ impl NetworkCallback {
                 ));
             }
 
-            if !tournament.has_started(Tick::now())
-                && tournament.has_available_spots()
+            if tournament.state(Tick::now()) == TournamentState::Registration
                 && !app.world.tournaments.contains_key(&tournament.id)
             {
-                let planet = app.world.get_planet_or_err(&tournament.planet_id)?;
-                app.ui.push_popup_to_top(PopupMessage::Ok {
+                let planet = app.world.planets.get_or_err(&tournament.planet_id)?;
+                app.ui.push_popup(PopupMessage::Ok {
                     message: format!(
                         "There is a tournament on {} starting in {} with available spots.",
                         planet.name,
-                        (tournament.starting_at - Tick::now()).formatted_as_time()
+                        (tournament.starting_at() - Tick::now()).formatted_as_time()
                     ),
                     is_skippable: true,
                     tick: Tick::now(),
                 });
             }
 
-            app.world
-                .tournaments
-                .insert(tournament.id, tournament.clone());
+            let received_tournament_is_latest =
+                if let Some(previous_t) = app.world.tournaments.get(&tournament.id) {
+                    // Try to keep most recent tournament. This is important to avoid recreating same games twice.
+                    // If previous tournament is already initialized, then no need to update
+                    if previous_t.is_initialized() {
+                        false
+                    } else {
+                        tournament.is_initialized()
+                            || tournament.is_canceled()
+                            || tournament.registered_teams.len() > previous_t.registered_teams.len()
+                    }
+                } else {
+                    true
+                };
+
+            if received_tournament_is_latest {
+                // We move tournament games to world to be able to siumulate them.
+                for game in tournament.games.iter() {
+                    if game.has_ended() && !app.world.past_games.contains_key(&game.id) {
+                        app.world
+                            .past_games
+                            .insert(game.id, GameSummary::from_game(game));
+                    } else if !game.has_ended() && !app.world.games.contains_key(&game.id) {
+                        app.world.games.insert(game.id, game.clone());
+                    }
+                }
+
+                app.world
+                    .tournaments
+                    .insert(tournament.id, tournament.clone());
+
+                app.world.dirty = true;
+                app.world.dirty_ui = true;
+            }
 
             Ok(None)
         })
@@ -445,7 +731,8 @@ impl NetworkCallback {
                         let mut trade = trade.clone();
                         let proposer_player = app
                             .world
-                            .get_player_or_err(&trade.proposer_player.id)?
+                            .players
+                            .get_or_err(&trade.proposer_player.id)?
                             .clone();
                         trade.proposer_player = proposer_player;
 
@@ -458,7 +745,7 @@ impl NetworkCallback {
                             .insert(trade.target_player.id, trade.target_player.clone());
 
                         let own_team = app.world.get_own_team()?;
-                        let target_team = app.world.get_team_or_err(
+                        let target_team = app.world.teams.get_or_err(
                             &trade
                                 .target_player
                                 .team
@@ -545,7 +832,7 @@ impl NetworkCallback {
                             .insert(trade.proposer_player.id, trade.proposer_player.clone());
 
                         let own_team = app.world.get_own_team()?;
-                        let proposer_team = app.world.get_team_or_err(
+                        let proposer_team = app.world.teams.get_or_err(
                             &trade
                                 .proposer_player
                                 .team
@@ -652,6 +939,8 @@ impl NetworkCallback {
 
                     let own_team = app.world.get_own_team_mut()?;
 
+                    // Here we check not to be committed to a tournament, to avoid accepting a challenge
+                    // which would stop the team from entering the tournament later.
                     if own_team.current_game.is_none()
                         && own_team.committed_to_tournament().is_none()
                         && own_team.autonomous_strategy.challenge_network
@@ -928,12 +1217,14 @@ impl NetworkCallback {
                     NetworkData::TournamentRegistrationRequest {
                         tournament_id,
                         team_id,
-                        state,
+                        team_data,
+                        request_state,
                         ..
                     } => Self::handle_tournament_registration_request_topic(
                         tournament_id,
                         team_id,
-                        state,
+                        team_data,
+                        request_state,
                     )(app),
                     NetworkData::Tournament { tournament, .. } => {
                         Self::handle_tournament_topic(tournament)(app)
