@@ -3,11 +3,10 @@ use super::trade::Trade;
 use crate::core::planet::Planet;
 use crate::core::position::{GamePosition, MAX_GAME_POSITION};
 use crate::core::skill::Skill;
-use crate::core::TournamentRegistrationState;
 use crate::game_engine::timer::Timer;
 use crate::game_engine::types::GameStats;
 use crate::game_engine::{Tournament, TournamentId};
-use crate::types::{PlanetId, PlayerId, Tick};
+use crate::types::{HashMapWithResult, PlanetId, PlayerId, PlayerMap, Tick};
 use crate::{
     core::{player::Player, team::Team, world::World},
     game_engine::types::TeamInGame,
@@ -21,6 +20,19 @@ use std::collections::HashMap;
 use std::fmt::Debug;
 use strum_macros::Display;
 
+#[derive(Debug, Display, Serialize, Deserialize, Clone, PartialEq)]
+pub enum TournamentRequestState {
+    RegistrationRequest,
+    RegistrationDeclined { reason: String },
+    RegistrationOk,
+    ConfirmationRequest,
+    ConfirmationDeclined { reason: String },
+    ParticipationRequest,
+    ParticipationDeclined { reason: String },
+    ParticipationOk,
+    Cancellation { reason: String },
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 #[allow(clippy::large_enum_variant)]
 pub enum NetworkData {
@@ -57,7 +69,8 @@ pub enum NetworkData {
         timestamp: Tick,
         tournament_id: TournamentId,
         team_id: TeamId,
-        state: TournamentRegistrationState,
+        team_data: Option<(Team, PlayerMap)>,
+        request_state: TournamentRequestState,
     },
     Tournament {
         timestamp: Tick,
@@ -79,12 +92,12 @@ pub enum NetworkRequestState {
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 pub struct NetworkTeam {
     pub team: Team,
-    pub players: Vec<Player>,
+    pub players: PlayerMap,
     pub asteroids: Vec<Planet>,
 }
 
 impl NetworkTeam {
-    pub fn new(team: Team, players: Vec<Player>, asteroids: Vec<Planet>) -> Self {
+    pub fn new(team: Team, players: PlayerMap, asteroids: Vec<Planet>) -> Self {
         Self {
             team,
             players,
@@ -93,14 +106,15 @@ impl NetworkTeam {
     }
 
     pub fn from_team_id(world: &World, team_id: &TeamId, peer_id: PeerId) -> AppResult<Self> {
-        let mut team = world.get_team_or_err(team_id)?.clone();
-        let mut players = world.get_players_by_team(&team)?;
+        let mut team = world.teams.get_or_err(team_id)?.clone();
+        let mut players = World::get_players_by_team(&world.players, &team)?;
         let asteroids = team
             .asteroid_ids
             .iter()
             .map(|asteroid_id| {
                 let mut asteroid = world
-                    .get_planet_or_err(asteroid_id)
+                    .planets
+                    .get_or_err(asteroid_id)
                     .expect("Asteroid should be part of world")
                     .clone();
                 asteroid.peer_id = Some(peer_id);
@@ -111,7 +125,7 @@ impl NetworkTeam {
         // Set the peer_id for team we are sending out
         // This means that the team can be challenged online and it will not be stored.
         team.peer_id = Some(peer_id);
-        for player in players.iter_mut() {
+        for player in players.values_mut() {
             player.peer_id = Some(peer_id);
         }
 
@@ -133,7 +147,7 @@ pub struct NetworkGame {
 
 impl NetworkGame {
     pub fn from_game_id(world: &World, game_id: &GameId) -> AppResult<Self> {
-        let game = world.get_game_or_err(game_id)?.clone();
+        let game = world.games.get_or_err(game_id)?.clone();
 
         let mut home_team_in_game = game.home_team_in_game.clone();
         // Reset stats
@@ -206,7 +220,7 @@ impl TeamRanking {
             timestamp,
             player_ratings: network_team
                 .players
-                .iter()
+                .values()
                 .map(|p| p.average_skill())
                 .collect_vec(),
         }
