@@ -15,8 +15,9 @@ mod traits;
 mod utils;
 mod visual_effects;
 
+use crate::core::{constants as core_constants, resources};
 pub use space::SpaceAdventure;
 pub use space_callback::SpaceCallback;
-pub use spaceship::{ShooterState, SpaceshipEntity};
+use spaceship::SpaceshipEntity;
 pub use traits::*;
-pub use utils::Direction;
+use utils::Direction;
