@@ -39,7 +39,7 @@ impl<'a> From<Button<'a>> for Text<'a> {
 
 impl<'a> Button<'a> {
     pub fn new(text: impl Into<Text<'a>>, on_click: UiCallback) -> Self {
-        Self::no_box(text.into(), on_click)
+        Self::no_box(text, on_click)
             .hover_block(default_block())
             .block(default_block())
     }
@@ -124,6 +124,10 @@ impl<'a> Button<'a> {
         self.layer = layer;
         self
     }
+
+    pub fn text_width(&self) -> usize {
+        self.text.width()
+    }
 }
 
 impl<'a> Styled for Button<'a> {
@@ -152,38 +156,38 @@ impl<'a> Widget for Button<'a> {
         };
 
         let paragraph = if let Some(u) = self.hotkey {
-            let split = self
-                .text
-                .to_string()
-                .splitn(2, &u.to_string())
-                .map(|s| s.to_string())
-                .collect::<Vec<String>>();
-            if split.len() > 1 {
-                Paragraph::new(Line::from(vec![
-                    Span::raw(split[0].clone()),
+            // The hotkey is displayed as an underscored character,
+            // only if it is on the first text line.
+            let first_line = &self.text.lines[0];
+            let first_line_string = first_line.to_string();
+
+            if let Some((before, after)) = first_line_string.split_once(&u.to_string()) {
+                let highlighted = Line::from(vec![
+                    Span::raw(before.to_owned()),
                     Span::styled(u.to_string(), UiStyle::DEFAULT.underlined()),
-                    Span::raw(split[1].clone()),
-                ]))
-                .alignment(self.text_alignemnt)
+                    Span::raw(after.to_owned()),
+                ]);
+
+                let mut lines = self.text.lines.clone();
+                lines[0] = highlighted;
+
+                Paragraph::new(lines)
             } else {
-                Paragraph::new(self.text.clone()).alignment(self.text_alignemnt)
+                Paragraph::new(self.text)
             }
         } else {
-            Paragraph::new(self.text.clone()).alignment(self.text_alignemnt)
-        };
-
-        let render_style = if self.is_hovered {
-            self.hover_style
-        } else {
-            self.style
-        };
+            Paragraph::new(self.text)
+        }
+        .alignment(self.text_alignemnt);
 
         let paragraph_style = if self.selected {
             UiStyle::SELECTED_BUTTON
         } else if self.disabled {
             UiStyle::UNSELECTABLE
+        } else if self.is_hovered {
+            self.hover_style
         } else {
-            render_style
+            self.style
         };
 
         let maybe_block = if self.is_hovered {
