@@ -12,6 +12,7 @@ use super::{
     widgets::{default_block, selectable_list, DOWN_ARROW_SPAN, SWITCH_ARROW_SPAN, UP_ARROW_SPAN},
 };
 use crate::store::load_game;
+use crate::types::HashMapWithResult;
 use crate::ui::traits::UiStyled;
 use crate::ui::ui_key;
 use crate::ui::widgets::GREEN_STYLE_SKILL;
@@ -53,7 +54,8 @@ pub struct GamePanel {
     pitch_view_filter: Option<Period>,
     player_status_view: bool,
     commentary_index: usize,
-    action_results: Vec<ActionOutput>,
+    // action_results: Vec<ActionOutput>,
+    action_results_len: usize,
     tick: usize,
     gif_map: GifMap,
 }
@@ -103,7 +105,7 @@ impl GamePanel {
         loaded_games: &'a HashMap<GameId, Game>,
     ) -> Option<&'a Game> {
         if let Some(game_id) = game_ids.get(index?) {
-            return world.get_game(game_id);
+            return world.games.get(game_id);
         }
 
         if let Some(game_id) = recent_game_ids.get(index?.checked_sub(game_ids.len())?) {
@@ -129,18 +131,29 @@ impl GamePanel {
         ])
         .split(area);
 
-        let game_button_split = Layout::vertical([
-            Constraint::Fill(1),
-            Constraint::Fill(1),
-            Constraint::Length(3),
-        ])
-        .split(split[0]);
-        self.build_game_list(frame, world, game_button_split[0]);
-        self.build_recent_games_list(frame, world, game_button_split[1]);
-        self.build_game_buttons(frame, game_button_split[2]);
+        let game_button_split =
+            Layout::vertical([Constraint::Fill(1), Constraint::Fill(1)]).split(split[0]);
+        frame.render_widget(
+            default_block().title("Past games ↓/↑"),
+            game_button_split[1],
+        );
+        frame.render_widget(default_block().title("Games ↓/↑"), split[0]);
+
+        self.build_game_list(frame, world, game_button_split[0].inner(Margin::new(1, 1)));
+        self.build_recent_games_list(frame, world, game_button_split[1].inner(Margin::new(1, 1)));
 
         self.build_score_panel(frame, world, split[1])?;
 
+        // Render game buttons on top of score panel.
+        let gbv_split = Layout::vertical([
+            Constraint::Fill(1),
+            Constraint::Length(3),
+            Constraint::Length(1),
+        ])
+        .split(split[1]);
+
+        self.build_game_buttons(frame, gbv_split[1]);
+
         Ok(())
     }
 
@@ -149,21 +162,15 @@ impl GamePanel {
             .game_ids
             .iter()
             .map(|id| {
-                let game = world.get_game(id)?;
+                let game = world.games.get(id)?;
                 let mut style = UiStyle::DEFAULT;
 
                 if game.home_team_in_game.team_id == world.own_team_id
                     || game.away_team_in_game.team_id == world.own_team_id
                 {
                     style = UiStyle::OWN_TEAM
-                } else {
-                    {
-                        if game.home_team_in_game.peer_id.is_some()
-                            || game.away_team_in_game.peer_id.is_some()
-                        {
-                            style = UiStyle::NETWORK;
-                        }
-                    }
+                } else if game.is_network() {
+                    style = UiStyle::NETWORK;
                 }
 
                 Some((
@@ -185,12 +192,10 @@ impl GamePanel {
             let list_index = self.index.filter(|&index| index < self.game_ids.len());
 
             frame.render_stateful_interactive_widget(
-                list.block(default_block().title("Games ↓/↑")),
+                list,
                 area,
                 &mut ClickableListState::default().with_selected(list_index),
             );
-        } else {
-            frame.render_widget(default_block().title("Games ↓/↑"), area);
         }
     }
 
@@ -244,18 +249,25 @@ impl GamePanel {
                 .map(|index| index - self.game_ids.len());
 
             frame.render_stateful_interactive_widget(
-                list.block(default_block().title("Past games ↓/↑")),
+                list,
                 area,
                 &mut ClickableListState::default().with_selected(list_index),
             );
-        } else {
-            frame.render_widget(default_block().title("Past games ↓/↑"), area);
         }
     }
 
     fn build_game_buttons(&mut self, frame: &mut UiFrame, area: Rect) {
-        let b_split =
-            Layout::horizontal([Constraint::Ratio(1, 2), Constraint::Ratio(1, 2)]).split(area);
+        if self.index.is_none() {
+            return;
+        };
+        let b_split = Layout::horizontal([
+            Constraint::Fill(3),
+            Constraint::Length(18),
+            Constraint::Fill(2),
+            Constraint::Length(18),
+            Constraint::Fill(3),
+        ])
+        .split(area);
         let text = if self.pitch_view {
             "Commentary view"
         } else {
@@ -272,7 +284,7 @@ impl GamePanel {
             ))
             .set_hotkey(ui_key::game::PITCH_VIEW);
 
-        frame.render_interactive_widget(pitch_button, b_split[0]);
+        frame.render_interactive_widget(pitch_button, b_split[1]);
 
         let text = if self.player_status_view {
             "Game stats"
@@ -290,7 +302,7 @@ impl GamePanel {
             ))
             .set_hotkey(ui_key::game::PLAYER_STATUS_VIEW);
 
-        frame.render_interactive_widget(player_status_button, b_split[1]);
+        frame.render_interactive_widget(player_status_button, b_split[3]);
     }
 
     fn build_score_panel(
@@ -321,12 +333,11 @@ impl GamePanel {
             Constraint::Length(PLAYER_IMAGE_WIDTH as u16),
             Constraint::Fill(1),
         ])
-        .split(area); //FIXME: check if split[0] is needed
+        .split(area);
 
         let central_split = Layout::vertical([
             Constraint::Fill(1),
-            Constraint::Length(1),
-            Constraint::Length(1),
+            Constraint::Length(2),
             Constraint::Length(2),
             Constraint::Length(8),
             Constraint::Fill(1),
@@ -343,7 +354,7 @@ impl GamePanel {
                 } else {
                     "Playing"
                 },
-                world.get_planet_or_err(&game.location)?.name,
+                world.planets.get_or_err(&game.location)?.name,
                 if let Some(timestamp) = game.ended_at {
                     format!(" on {}", timestamp.formatted_as_date())
                 } else {
@@ -351,16 +362,16 @@ impl GamePanel {
                 }
             ))
             .centered(),
-            central_split[3],
+            central_split[2],
         );
 
         let digit_split =
-            Layout::horizontal([8, 1, 8, 1, 8, 1, 5, 1, 8, 1, 8, 1, 8]).split(central_split[4]);
+            Layout::horizontal([8, 1, 8, 1, 8, 1, 5, 1, 8, 1, 8, 1, 8]).split(central_split[3]);
 
         let action = if self.commentary_index == 0 {
             &game.action_results[game.action_results.len() - 1]
         } else {
-            &game.action_results[self.action_results.len() - 1 - self.commentary_index]
+            &game.action_results[self.action_results_len - 1 - self.commentary_index]
         };
         let home_score = action.home_score;
         let away_score = action.away_score;
@@ -405,28 +416,65 @@ impl GamePanel {
         }
 
         let home_dot = if action.possession == Possession::Home {
-            "●"
+            "● "
         } else {
-            " "
+            "  "
         };
         let away_dot = if action.possession == Possession::Away {
-            "●"
+            " ●"
         } else {
-            " "
+            "  "
         };
         let l = MAX_NAME_LENGTH + 2;
-        frame.render_widget(
-            Paragraph::new(Line::from(format!(
-                "{:>l$} vs {:<l$}",
-                format!("{} {}", home_dot, game.home_team_in_game.name),
-                format!("{} {}", game.away_team_in_game.name, away_dot),
-            )))
-            .centered(),
-            central_split[1],
-        );
+        let spans_split = Layout::horizontal([
+            Constraint::Fill(1),
+            Constraint::Length(2),
+            Constraint::Max(l as u16),
+            Constraint::Length(4),
+            Constraint::Max(l as u16),
+            Constraint::Length(2),
+            Constraint::Fill(1),
+        ])
+        .split(central_split[1]);
+        frame.render_widget(Paragraph::new(home_dot), spans_split[1]);
+
+        if world.teams.contains_key(&game.home_team_in_game.team_id) {
+            let home_button = Button::new(
+                format!("{:>}", game.home_team_in_game.name),
+                UiCallback::GoToTeam {
+                    team_id: game.home_team_in_game.team_id,
+                },
+            )
+            .set_hover_text(format!("Go to {} team", game.home_team_in_game.name));
+            frame.render_interactive_widget(home_button, spans_split[2]);
+        } else {
+            frame.render_widget(
+                Paragraph::new(format!("{:>}", game.home_team_in_game.name)),
+                spans_split[2],
+            );
+        }
+
+        frame.render_widget(Paragraph::default(), spans_split[3]);
+
+        if world.teams.contains_key(&game.away_team_in_game.team_id) {
+            let away_button = Button::new(
+                format!("{:<}", game.away_team_in_game.name),
+                UiCallback::GoToTeam {
+                    team_id: game.away_team_in_game.team_id,
+                },
+            )
+            .set_hover_text(format!("Go to {} team", game.away_team_in_game.name));
+            frame.render_interactive_widget(away_button, spans_split[4]);
+        } else {
+            frame.render_widget(
+                Paragraph::new(format!("{:<}", game.away_team_in_game.name)),
+                spans_split[4],
+            );
+        }
+        frame.render_widget(Paragraph::new(away_dot), spans_split[5]);
 
         let timer_lines = self.build_timer_lines(world, game);
-        frame.render_widget(Paragraph::new(timer_lines).centered(), central_split[5]);
+        frame.render_widget(Paragraph::new(timer_lines).centered(), central_split[4]);
         match home_score {
             x if x < 10 => frame.render_widget((home_score % 10).big_font(), digit_split[4]),
             x if x < 100 => {
@@ -472,7 +520,7 @@ impl GamePanel {
     ) -> AppResult<()> {
         frame.render_widget(default_block().title("Shots map"), area);
 
-        let planet = world.get_planet_or_err(&game.location)?;
+        let planet = world.planets.get_or_err(&game.location)?;
         let pitch_style = match planet.planet_type {
             PlanetType::Earth => PitchImage::PitchFancy,
             PlanetType::Ring | PlanetType::Rocky => PitchImage::PitchBall,
@@ -480,7 +528,7 @@ impl GamePanel {
             _ => PitchImage::PitchClassic,
         };
 
-        let max_index = self.action_results.len() - self.commentary_index;
+        let max_index = self.action_results_len - self.commentary_index;
 
         // These map will contain every shot up to the max_index action.
         let mut shots_map: HashMap<(u32, u32), (u8, u8)> = HashMap::new();
@@ -614,7 +662,7 @@ impl GamePanel {
         } else if self.pitch_view {
             self.build_pitch_panel(frame, world, game, split[0])?;
         } else {
-            self.build_commentary(frame, split[0]);
+            self.build_commentary(frame, game, split[0]);
         }
 
         if self.player_status_view {
@@ -646,16 +694,21 @@ impl GamePanel {
         Line::from(vec![timer, text, arrow])
     }
 
-    fn build_commentary(&self, frame: &mut UiFrame, area: Rect) {
+    fn build_commentary(&self, frame: &mut UiFrame, game: &Game, area: Rect) {
         let mut commentary = vec![];
-        let max_index = self.action_results.len() - self.commentary_index;
+        let max_index = self
+            .action_results_len
+            .saturating_sub(self.commentary_index)
+            // This is necessary as action_results_len is updated in the update
+            // but the game could be modified by changin the index.
+            .min(game.action_results.len());
 
         for idx in 0..max_index {
-            let result = &self.action_results[idx];
+            let result = &game.action_results[idx];
             let situation = &result.situation;
-            let timer = self.action_results[idx].start_at;
+            let timer = game.action_results[idx].start_at;
             let switch_possession = if idx > 0 {
-                result.possession != self.action_results[idx - 1].possession
+                result.possession != game.action_results[idx - 1].possession
             } else {
                 false
             };
@@ -870,7 +923,7 @@ impl GamePanel {
 
     fn build_timer_lines(&self, world: &World, game: &Game) -> Vec<Line<'static>> {
         let timer = if self.commentary_index > 0 {
-            self.action_results[self.action_results.len() - 1 - self.commentary_index].start_at
+            game.action_results[self.action_results_len - 1 - self.commentary_index].start_at
         } else {
             game.timer
         };
@@ -1128,11 +1181,7 @@ impl Screen for GamePanel {
             &self.loaded_game_ids,
             &self.loaded_games,
         ) {
-            if self.commentary_index == 0 && self.action_results.len() != game.action_results.len()
-            {
-                // FIXME: we dont need to clone, remove self.action_results completely
-                self.action_results = game.action_results.clone();
-            }
+            self.action_results_len = game.action_results.len();
         }
 
         Ok(())
@@ -1171,7 +1220,7 @@ impl Screen for GamePanel {
                 }
             }
             ui_key::NEXT_SELECTION => {
-                if self.commentary_index < self.action_results.len() - 1 {
+                if self.commentary_index < self.action_results_len - 1 {
                     self.commentary_index += 1;
                 }
             }
