@@ -11,7 +11,7 @@ use super::{
     widgets::*,
 };
 use crate::game_engine::timer::Period;
-use crate::types::Tick;
+use crate::types::{HashMapWithResult, Tick};
 use crate::ui::popup_message::PopupMessage;
 use crate::ui::ui_key;
 use crate::{
@@ -217,7 +217,7 @@ impl MyTeamPanel {
 
         let mut options = vec![];
         for id in self.planet_markets.iter() {
-            let planet = world.get_planet_or_err(id)?;
+            let planet = world.planets.get_or_err(id)?;
             let text = planet.name.clone();
             let style = match own_team.current_location {
                 TeamLocation::OnPlanet { planet_id } => {
@@ -244,7 +244,7 @@ impl MyTeamPanel {
 
         let planet_id =
             self.planet_markets[self.planet_index.unwrap_or_default() % self.planet_markets.len()];
-        let planet = world.get_planet_or_err(&planet_id)?;
+        let planet = world.planets.get_or_err(&planet_id)?;
         let merchant_bonus = TeamBonus::TradePrice.current_team_bonus(world, &own_team.id)?;
 
         let mut lines = vec![Line::from(Span::styled(
@@ -320,7 +320,7 @@ impl MyTeamPanel {
             }
         };
 
-        let planet = world.get_planet_or_err(&planet_id)?;
+        let planet = world.planets.get_or_err(&planet_id)?;
         frame.render_widget(
             default_block().title(format!("Planet {} Market", planet.name)),
             area,
@@ -791,7 +791,7 @@ impl MyTeamPanel {
             .take(displayed_challenges)
             .enumerate()
         {
-            let team = world.get_team_or_err(team_id)?;
+            let team = world.teams.get_or_err(team_id)?;
             frame.render_widget(
                 Paragraph::new(format!(
                     "{:<MAX_NAME_LENGTH$} {}",
@@ -826,7 +826,7 @@ impl MyTeamPanel {
 
         let mut options = vec![];
         if let Some(game_id) = own_team.current_game {
-            if let Ok(game) = world.get_game_or_err(&game_id) {
+            if let Ok(game) = world.games.get_or_err(&game_id) {
                 if let Some(action) = game.action_results.last() {
                     let text = format!(
                         " {:>12} {:>3}-{:<3} {:<}",
@@ -917,14 +917,16 @@ impl MyTeamPanel {
             frame.render_interactive_widget(button, v_split[1]);
         }
 
-        let summary = if let Ok(current_game) = world.get_game_or_err(&game_id) {
+        let summary = if let Ok(current_game) = world.games.get_or_err(&game_id) {
             let (home_quarters_score, away_quarters_score) = current_game.get_score_by_quarter();
 
             let lines = vec![
                 Line::from(format!(
                     "Location {} - Attendance {}",
-                    if let Ok(planet) = world.get_planet_or_err(&current_game.location) {
+                    if let Ok(planet) = world.planets.get_or_err(&current_game.location) {
                         planet.name.as_str()
+                    } else if current_game.planet_name != String::default() {
+                        current_game.planet_name.as_str()
                     } else {
                         "Unknown"
                     },
@@ -1018,8 +1020,10 @@ impl MyTeamPanel {
             let mut lines = vec![
                 Line::from(format!(
                     "Location {} - Attendance {}",
-                    if let Ok(planet) = world.get_planet_or_err(&game_summary.location) {
+                    if let Ok(planet) = world.planets.get_or_err(&game_summary.location) {
                         planet.name.as_str()
+                    } else if game_summary.planet_name != String::default() {
+                        game_summary.planet_name.as_str()
                     } else {
                         "Unknown"
                     },
@@ -1382,7 +1386,8 @@ impl MyTeamPanel {
 
         frame.render_widget(Paragraph::new(lines), h_split[1].inner(Margin::new(3, 1)));
 
-        Self::render_available_spaceship_upgrades(
+        render_available_spaceship_upgrades(
+            own_team.spaceship.pending_upgrade,
             possible_upgrade,
             world,
             own_team,
@@ -1394,46 +1399,6 @@ impl MyTeamPanel {
         Ok(())
     }
 
-    fn render_available_spaceship_upgrades(
-        possible_upgrade: Option<Upgrade<SpaceshipUpgradeTarget>>,
-        world: &World,
-        own_team: &Team,
-        frame: &mut UiFrame,
-        area: Rect,
-    ) -> AppResult<()> {
-        let spaceship = &own_team.spaceship;
-        if let Some(pending_upgrade) = spaceship.pending_upgrade {
-            let header = match pending_upgrade.target {
-                SpaceshipUpgradeTarget::Repairs { .. } => "Repairing spaceship".to_string(),
-                target => format!("Upgrading {target}"),
-            };
-
-            let countdown = (pending_upgrade.started + pending_upgrade.duration)
-                .saturating_sub(world.last_tick_short_interval)
-                .formatted();
-
-            frame.render_widget(
-                Paragraph::new(vec![
-                    Line::from(vec![Span::styled(header, UiStyle::HEADER.bold())]),
-                    Line::from(countdown),
-                ])
-                .centered(),
-                area,
-            )
-        } else if let Some(upgrade) = possible_upgrade {
-            {
-                let mut lines =
-                    vec![
-                        Line::from(Span::styled("Upgrade cost", UiStyle::HEADER.bold())).centered(),
-                    ];
-                lines.append(&mut upgrade_resources_lines(upgrade.target, own_team));
-                frame.render_widget(Paragraph::new(lines).centered(), area);
-            }
-        }
-
-        Ok(())
-    }
-
     fn render_upgrade_spaceship_button(
         &self,
         possible_upgrade: Option<Upgrade<SpaceshipUpgradeTarget>>,
@@ -1558,9 +1523,9 @@ impl MyTeamPanel {
         let options = self
             .asteroid_ids
             .iter()
-            .filter(|&asteroid_id| world.get_planet_or_err(asteroid_id).is_ok())
+            .filter(|&asteroid_id| world.planets.get_or_err(asteroid_id).is_ok())
             .map(|&asteroid_id| {
-                let asteroid = world.get_planet_or_err(&asteroid_id).unwrap();
+                let asteroid = world.planets.get_or_err(&asteroid_id).unwrap();
                 let style = match own_team.current_location {
                     TeamLocation::OnPlanet { planet_id } => {
                         if planet_id == asteroid_id {
@@ -1589,7 +1554,7 @@ impl MyTeamPanel {
 
         if let Some(index) = self.asteroid_index {
             let asteroid_id = own_team.asteroid_ids[index % own_team.asteroid_ids.len()];
-            let asteroid = world.get_planet_or_err(&asteroid_id)?;
+            let asteroid = world.planets.get_or_err(&asteroid_id)?;
 
             let mut lines = vec![Line::from(Span::styled(
                 "Resources",
@@ -1630,15 +1595,15 @@ impl MyTeamPanel {
                 None
             };
 
-            Self::render_available_asteroid_upgrades(
-                asteroid,
-                world,
+            render_available_upgrades(
+                asteroid.pending_upgrade,
                 possible_upgrade,
+                world,
                 own_team,
                 frame,
                 v_split[1],
             )?;
-            Self::render_build_asteroid_upgrade_button(
+            render_build_asteroid_upgrade_button(
                 asteroid,
                 possible_upgrade,
                 own_team,
@@ -1650,118 +1615,6 @@ impl MyTeamPanel {
         Ok(())
     }
 
-    fn render_available_asteroid_upgrades(
-        asteroid: &Planet,
-        world: &World,
-        possible_upgrade: Option<Upgrade<AsteroidUpgradeTarget>>,
-        own_team: &Team,
-        frame: &mut UiFrame,
-        area: Rect,
-    ) -> AppResult<()> {
-        if let Some(pending_upgrade) = asteroid.pending_upgrade {
-            let countdown = (pending_upgrade.started + pending_upgrade.duration)
-                .saturating_sub(world.last_tick_short_interval)
-                .formatted();
-
-            frame.render_widget(
-                Paragraph::new(vec![
-                    Line::from(Span::styled(
-                        format!("Building {}", pending_upgrade.target),
-                        UiStyle::HEADER.bold(),
-                    )),
-                    Line::from(countdown),
-                ])
-                .centered(),
-                area,
-            );
-        } else if let Some(upgrade) = possible_upgrade {
-            {
-                let mut lines = vec![Line::from(Span::styled(
-                    format!("{} upgrade cost", upgrade.target),
-                    UiStyle::HEADER.bold(),
-                ))];
-                lines.append(&mut upgrade_resources_lines(upgrade.target, own_team));
-                frame.render_widget(Paragraph::new(lines).centered(), area);
-            }
-        } else {
-            let lines = vec![
-                Line::from("Nothing left"),
-                Line::from(format!("to build on {}", asteroid.name)),
-            ];
-            frame.render_widget(Paragraph::new(lines).centered(), area);
-        }
-
-        Ok(())
-    }
-
-    fn render_build_asteroid_upgrade_button(
-        asteroid: &Planet,
-        possible_upgrade: Option<Upgrade<AsteroidUpgradeTarget>>,
-        own_team: &Team,
-        frame: &mut UiFrame,
-        area: Rect,
-    ) -> AppResult<()> {
-        if let Some(pending_upgrade) = asteroid.pending_upgrade {
-            let build_button = Button::new(
-                format!("Building {}", pending_upgrade.target),
-                UiCallback::None,
-            )
-            .set_hover_text(format!(
-                "Building {} on {}",
-                pending_upgrade.target, asteroid.name
-            ))
-            .disabled(Some(format!("Already building {}", pending_upgrade.target)));
-
-            frame.render_interactive_widget(build_button, area);
-        } else if let Some(upgrade) = possible_upgrade {
-            let on_click = if upgrade.target == AsteroidUpgradeTarget::SpaceCove {
-                UiCallback::PushUiPopup {
-                    popup_message: PopupMessage::BuildSpaceCove {
-                        asteroid_name: asteroid.name.clone(),
-                        asteroid_id: asteroid.id,
-                        tick: Tick::now(),
-                    },
-                }
-            } else {
-                UiCallback::SetAsteroidPendingUpgrade {
-                    asteroid_id: asteroid.id,
-                    upgrade,
-                }
-            };
-
-            let mut build_button = Button::new(
-                format!(
-                    "Build {} ({})",
-                    upgrade.target,
-                    upgrade.duration.formatted()
-                ),
-                on_click,
-            )
-            .set_hotkey(ui_key::BUILD_ASTEROID_UPGRADE)
-            .set_hover_text(upgrade.target.description());
-
-            if upgrade.target == AsteroidUpgradeTarget::SpaceCove {
-                build_button = build_button.block(default_block().border_style(UiStyle::WARNING));
-            }
-
-            let can_upgrade_asteroid = own_team.can_upgrade_asteroid(asteroid, &upgrade);
-            if let Err(e) = can_upgrade_asteroid.as_ref() {
-                build_button.disable(Some(e.to_string()));
-            }
-
-            frame.render_interactive_widget(build_button, area);
-        } else {
-            let build_button = Button::new(
-                format!("Nothing left to build on {}", asteroid.name),
-                UiCallback::None,
-            )
-            .disabled(None::<String>);
-            frame.render_interactive_widget(build_button, area);
-        }
-
-        Ok(())
-    }
-
     fn render_selected_asteroid(
         &mut self,
         frame: &mut UiFrame,
@@ -1775,12 +1628,12 @@ impl MyTeamPanel {
 
         let asteroid_id =
             self.asteroid_ids[self.asteroid_index.unwrap_or_default() % self.asteroid_ids.len()];
-        let asteroid = world.get_planet_or_err(&asteroid_id)?;
+        let asteroid = world.planets.get_or_err(&asteroid_id)?;
 
         let mut parents = vec![asteroid];
         let mut current = asteroid;
         while let Some(parent_id) = current.satellite_of {
-            let parent = world.get_planet_or_err(&parent_id)?;
+            let parent = world.planets.get_or_err(&parent_id)?;
             parents.push(parent);
             current = parent;
         }
@@ -2182,7 +2035,7 @@ impl MyTeamPanel {
         let sorted_players = own_team
             .player_ids
             .iter()
-            .map(|id| world.get_player(id).unwrap())
+            .map(|id| world.players.get(id).unwrap())
             .collect_vec()
             .sort_by_rating();
 
@@ -2218,7 +2071,7 @@ impl MyTeamPanel {
         );
 
         if let Some(game_id) = own_team.current_game {
-            let game = world.get_game_or_err(&game_id)?;
+            let game = world.games.get_or_err(&game_id)?;
             let game_text = format!(
                 "{:>} {:>3}-{:<3} {:<}",
                 game.home_team_in_game.name,
@@ -2474,7 +2327,7 @@ impl MyTeamPanel {
             let paragraph = Paragraph::new(lines);
             frame.render_widget(paragraph.centered(), rect);
         }
-        let planet = world.get_planet_or_err(planet_id)?;
+        let planet = world.planets.get_or_err(planet_id)?;
         frame.render_widget(
             default_block().title(format!("Travelling to {} - {}", planet.name, countdown)),
             area,
@@ -2512,7 +2365,7 @@ impl MyTeamPanel {
             let paragraph = Paragraph::new(lines);
             frame.render_widget(paragraph.centered(), rect);
         }
-        let planet = world.get_planet_or_err(planet_id)?;
+        let planet = world.planets.get_or_err(planet_id)?;
         frame.render_widget(
             default_block().title(format!("Exploring around {} - {}", planet.name, countdown)),
             area,
@@ -2603,7 +2456,7 @@ impl Screen for MyTeamPanel {
                 .teams
                 .keys()
                 .filter(|&id| {
-                    let team = if let Ok(team) = world.get_team_or_err(id) {
+                    let team = if let Ok(team) = world.teams.get_or_err(id) {
                         team
                     } else {
                         return false;
@@ -2614,8 +2467,8 @@ impl Screen for MyTeamPanel {
                 .cloned()
                 .collect();
             self.challenge_teams.sort_by(|a, b| {
-                let a = world.get_team_or_err(a).unwrap();
-                let b = world.get_team_or_err(b).unwrap();
+                let a = world.teams.get_or_err(a).unwrap();
+                let b = world.teams.get_or_err(b).unwrap();
                 world
                     .team_rating(&b.id)
                     .unwrap_or_default()
