@@ -5,17 +5,21 @@ use super::ui_callback::UiCallback;
 use super::ui_frame::UiFrame;
 use super::utils::{img_to_lines, input_from_key_event, validate_textarea_input};
 use super::widgets::{default_block, thick_block};
-use crate::image::types::{Gif, PrintableGif};
+use crate::core::planet::PlanetType;
+use crate::core::{player::Player, resources::Resource, skill::Rated};
+use crate::image::utils::open_gif;
 use crate::types::*;
+use crate::ui::constants::MAX_NAME_LENGTH;
 use crate::ui::gif_map::PORTAL_GIFS;
+use crate::ui::traits::PrintableGif;
 use crate::ui::ui_key;
-use crate::world::planet::PlanetType;
-use crate::world::{player::Player, resources::Resource, skill::Rated};
 use anyhow::anyhow;
 use core::fmt::Debug;
+use itertools::Itertools;
 use ratatui::layout::{Constraint, Layout};
 use ratatui::layout::{Margin, Rect};
 use ratatui::style::Stylize;
+use ratatui::text::Line;
 use ratatui::widgets::{Clear, Paragraph, Wrap};
 use strum_macros::Display;
 use tui_textarea::TextArea;
@@ -36,7 +40,6 @@ pub enum PopupMessage {
     },
     PromptQuit {
         during_space_adventure: bool,
-        on_peer_asteroid: bool,
         tick: Tick,
     },
     ReleasePlayer {
@@ -84,7 +87,7 @@ pub enum PopupMessage {
 }
 
 impl PopupMessage {
-    const MAX_TUTORIAL_PAGE: usize = 3;
+    const MAX_TUTORIAL_PAGE: usize = 4;
     fn rect(&self, area: Rect) -> Rect {
         let (width, height) = match self {
             PopupMessage::AsteroidNameDialog { .. } => (54, 28),
@@ -150,6 +153,7 @@ impl PopupMessage {
                         .chars()
                         .enumerate()
                         .map(|(i, c)| if i == 0 { c.to_ascii_uppercase() } else { c })
+                        .take(MAX_NAME_LENGTH)
                         .collect();
                     if validate_textarea_input(popup_input, "Asteroid name") {
                         let filename = format!("asteroid{}", tick % 30);
@@ -256,10 +260,10 @@ impl PopupMessage {
                     .centered(),
                     split[0],
                 );
+
+                let lines = message.split("\n").map(Line::from).collect_vec();
                 frame.render_widget(
-                    Paragraph::new(message.clone())
-                        .centered()
-                        .wrap(Wrap { trim: true }),
+                    Paragraph::new(lines).centered().wrap(Wrap { trim: true }),
                     split[1].inner(Margin {
                         horizontal: 1,
                         vertical: 1,
@@ -474,7 +478,6 @@ impl PopupMessage {
 
             PopupMessage::PromptQuit {
                 during_space_adventure,
-                on_peer_asteroid,
                 ..
             } => {
                 frame.render_widget(
@@ -487,8 +490,6 @@ impl PopupMessage {
 
                 let text = if *during_space_adventure {
                     format!("Are you sure you want to quit?\nYou will lose the whole cargo! Go back to the base first\n(Press '{}')", ui_key::space::BACK_TO_BASE)
-                } else if *on_peer_asteroid {
-                    "Are you sure you want to quit?\nYou will lose all your rum! Travel back to a planet first".to_string()
                 } else {
                     "Are you sure you want to quit?".to_string()
                 };
@@ -813,7 +814,7 @@ impl PopupMessage {
                 let planet_gif = if *planet_type == PlanetType::Asteroid {
                     GifMap::asteroid_zoom_out(planet_filename)?
                 } else {
-                    Gif::open(format!("planets/{planet_filename}_zoomout.gif"))?
+                    open_gif(format!("planets/{planet_filename}_zoomout.gif"))?
                 };
 
                 let planet_gif_lines = planet_gif.to_lines();
@@ -879,10 +880,11 @@ impl PopupMessage {
                 );
 
                 let message = match index {
-                    0 => "Hello pirate! This is your team page.\nHere you can check your pirates and ship and interact with the market.",
-                    1 => "You can navigate around by clicking on the tabs\nor using the arrow keys.",
+                    0 => "Hello pirate! This is your team page.\nHere you can check your pirates, upgrade your spaceship and trade resources on the market.",
+                    1 => "You can navigate around by clicking on the tabs at the top\nor using the arrow keys.",
                     2 => "To start, you can try to challenge another team to a game,\nor maybe explore around your planet to gather resources.",
-                    _ => "Ultimately, the key is to gather together a great crew.\nHave fun!"
+                    3 => "Ultimately, the key is to gather together a great crew and mess around.\n",
+                    _ => "Be sure to check out the Chat in the Swarm panel from time to time.\nHave fun!"
                 };
 
                 frame.render_widget(
