@@ -1,13 +1,16 @@
 use super::ui_frame::UiFrame;
 use super::{traits::Screen, ui_callback::UiCallback};
+use crate::game_engine::TournamentType;
 use crate::image::utils::ExtraImageUtils;
 use crate::image::utils::{open_image, LightMaskStyle};
-use crate::types::{SystemTimeTick, TeamId, Tick};
+use crate::types::{HashMapWithResult, TeamId};
 use crate::ui::button::Button;
 use crate::ui::clickable_list::ClickableListState;
 use crate::ui::traits::SplitPanel;
 use crate::ui::utils::img_to_lines;
-use crate::ui::widgets::{default_block, selectable_list, teleport_button};
+use crate::ui::widgets::{
+    default_block, render_available_upgrades, selectable_list, teleport_button,
+};
 use crate::ui::{constants::*, ui_key};
 use crate::{core::*, types::AppResult};
 use anyhow::anyhow;
@@ -18,6 +21,7 @@ use ratatui::layout::{Constraint, Layout, Margin};
 use ratatui::prelude::Rect;
 use ratatui::style::Stylize;
 use ratatui::widgets::Paragraph;
+use std::collections::HashSet;
 
 #[derive(Debug, Default)]
 pub struct SpaceCovePanel {
@@ -51,15 +55,18 @@ impl SpaceCovePanel {
 
     fn get_asteroid(world: &World) -> AppResult<&Planet> {
         let own_team = world.get_own_team()?;
-        let asteroid_id = match own_team.space_cove {
-            SpaceCoveState::Ready { planet_id } => planet_id,
-            state => {
-                return Err(anyhow!(
-                    "Space cove panel should not exist for space cove state {state}."
-                ))
-            }
-        };
-        world.get_planet_or_err(&asteroid_id)
+        let asteroid_id = own_team
+            .has_space_cove_on()
+            .ok_or_else(|| anyhow!("Space cove panel should be ready."))?;
+        world.planets.get_or_err(&asteroid_id)
+    }
+
+    fn _get_space_cove(own_team: &Team) -> AppResult<&SpaceCove> {
+        own_team
+            .space_cove
+            .as_ref()
+            .filter(|cove| cove.state == SpaceCoveState::Ready)
+            .ok_or_else(|| anyhow!("Space cove panel should be ready."))
     }
 
     fn get_cove_images(
@@ -88,12 +95,15 @@ impl SpaceCovePanel {
             base.copy_non_trasparent_from(&right_eye, RIGHT_EYE_POSITION.0, RIGHT_EYE_POSITION.1)?;
         }
 
-        let mut x = 7;
+        let mut x = 5;
         for team in teams.iter().take(4) {
             let ship_img = &team.spaceship.compose_image_in_shipyard()?[0];
             let y = 40;
             base.copy_non_trasparent_from(ship_img, x, y)?;
-            x += ship_img.width() + 2;
+            x += ship_img.width();
+            if x + ship_img.width() > base.width() {
+                break;
+            }
         }
 
         if !is_blinking_left {
@@ -129,14 +139,13 @@ impl SpaceCovePanel {
     fn render_visiting_teams(
         &self,
         frame: &mut UiFrame,
-        asteroid: &Planet,
         world: &World,
         area: Rect,
     ) -> AppResult<()> {
-        if !asteroid.team_ids.is_empty() {
+        if !self.team_ids.is_empty() {
             let mut options = vec![];
-            for team_id in asteroid.team_ids.iter() {
-                let team = if let Some(team) = world.get_team(team_id) {
+            for team_id in self.team_ids.iter() {
+                let team = if let Some(team) = world.teams.get(team_id) {
                     team
                 } else {
                     continue;
@@ -168,30 +177,73 @@ impl SpaceCovePanel {
         Ok(())
     }
 
-    fn render_tournament_button(
+    fn render_tournament_buttons(
         &self,
         frame: &mut UiFrame,
         asteroid: &Planet,
         world: &World,
         area: Rect,
     ) -> AppResult<()> {
+        let split = Layout::vertical([3, 3]).split(area);
         let own_team = world.get_own_team()?;
         let mut tournament_button = Button::new(
-            "Organize tournament",
+            "Organize quick tournament",
             UiCallback::OrganizeNewTournament {
-                max_participants: 4,
-                starting_at: Tick::now() + HOURS,
+                tournament_type: TournamentType::Cup
             },
         )
-        .set_hotkey(ui_key::ORGANIZE_TOURNAMENT)
-        .set_hover_text(format!("Organize a tournament on {}", asteroid.name));
+        .set_hotkey(ui_key::ORGANIZE_QUICK_TOURNAMENT)
+        .set_hover_text(format!("Organize a quick tournament on {}. Registrations close in 5 minutes, max 4 participants.", asteroid.name));
 
         if let Err(err) = own_team.can_organize_tournament() {
             tournament_button.disable(Some(err.to_string()));
         }
 
-        tournament_button.disable(Some("Coming soon!"));
-        frame.render_interactive_widget(tournament_button, area);
+        frame.render_interactive_widget(tournament_button, split[0]);
+
+        let mut tournament_button = Button::new(
+            "Organize big tournament",
+            UiCallback::OrganizeNewTournament {
+                tournament_type: TournamentType::Supercup,
+            },
+        )
+        .set_hotkey(ui_key::ORGANIZE_BIG_TOURNAMENT)
+        .set_hover_text(format!(
+            "Organize a big tournament on {}. Registrations close in 1 hour, max 8 participants.",
+            asteroid.name
+        ));
+
+        if let Err(err) = own_team.can_organize_tournament() {
+            tournament_button.disable(Some(err.to_string()));
+        }
+
+        frame.render_interactive_widget(tournament_button, split[1]);
+
+        Ok(())
+    }
+
+    fn _render_upgrades(&self, frame: &mut UiFrame, world: &World, area: Rect) -> AppResult<()> {
+        let own_team = world.get_own_team()?;
+        let space_cove = Self::_get_space_cove(own_team)?;
+
+        let split = Layout::vertical([Constraint::Fill(1), Constraint::Length(4)]).split(area);
+
+        let bonus = TeamBonus::Upgrades.current_team_bonus(world, &own_team.id)?;
+        let possible_upgrade = if !space_cove.upgrades.contains(&SpaceCoveUpgradeTarget::Skull) {
+            Some(Upgrade::new(SpaceCoveUpgradeTarget::Skull, bonus))
+        } else {
+            None
+        };
+
+        frame.render_widget(default_block(), split[0]);
+        render_available_upgrades::<SpaceCoveUpgradeTarget>(
+            space_cove.pending_upgrade,
+            possible_upgrade,
+            world,
+            own_team,
+            frame,
+            split[0].inner(Margin::new(1, 1)),
+        )?;
 
         Ok(())
     }
@@ -203,14 +255,21 @@ impl Screen for SpaceCovePanel {
 
         let asteroid = Self::get_asteroid(world)?;
         if world.dirty_ui || self.team_ids.len() != asteroid.team_ids.len() {
-            self.team_ids = asteroid.team_ids.clone();
+            // Update team_ids but maintain team_ids existing order.
+            let new_set: HashSet<TeamId> = asteroid.team_ids.iter().copied().collect();
+            self.team_ids.retain(|id| new_set.contains(id));
+            for id in &asteroid.team_ids {
+                if !self.team_ids.contains(id) {
+                    self.team_ids.push(*id);
+                }
+            }
 
             let teams = self
                 .team_ids
                 .iter()
                 .take(4)
                 .filter(|id| world.teams.contains_key(id))
-                .map(|id| world.get_team(id).unwrap())
+                .map(|id| world.teams.get(id).unwrap())
                 .collect_vec();
 
             let cove_image_widget = Self::get_cove_image_widgets(&teams, false, false)
@@ -258,7 +317,7 @@ impl Screen for SpaceCovePanel {
         } else if !left_eye_blinking && right_eye_blinking {
             &self.cove_image_widgets[2]
         } else {
-            //if left_eye_blinking && right_eye_blinking
+            //if both left_eye_blinking and right_eye_blinking
             &self.cove_image_widgets[3]
         };
 
@@ -269,8 +328,9 @@ impl Screen for SpaceCovePanel {
         let side_split = Layout::vertical([
             Constraint::Length(3),
             Constraint::Length(3),
-            Constraint::Length(3),
+            Constraint::Length(6),
             Constraint::Length(8),
+            Constraint::Length(11),
             Constraint::Fill(1),
         ])
         .split(split[0]);
@@ -283,14 +343,9 @@ impl Screen for SpaceCovePanel {
         );
 
         frame.render_interactive_widget(teleport_button(world, asteroid.id)?, side_split[1]);
-        self.render_tournament_button(frame, asteroid, world, side_split[2])?;
-
-        self.render_visiting_teams(frame, asteroid, world, side_split[3])?;
-
-        frame.render_widget(
-            default_block().title("No available upgrades"),
-            side_split[4],
-        );
+        self.render_tournament_buttons(frame, asteroid, world, side_split[2])?;
+        self.render_visiting_teams(frame, world, side_split[3])?;
+        // self.render_upgrades(frame, world, side_split[4])?;
 
         Ok(())
     }
