@@ -15,7 +15,7 @@ use super::{
 use crate::core::constants::MIN_PLAYERS_PER_GAME;
 use crate::core::team::Team;
 use crate::image::spaceship::{SPACESHIP_IMAGE_HEIGHT, SPACESHIP_IMAGE_WIDTH};
-use crate::types::AppResult;
+use crate::types::{AppResult, HashMapWithResult};
 use crate::ui::ui_key;
 use crate::{
     core::{
@@ -133,7 +133,7 @@ impl TeamListPanel {
         )
         .bold()
         .set_hotkey(ui_key::CYCLE_VIEW)
-        .set_hover_text("View all teams.");
+        .set_hover_text("View all crews.");
 
         let mut filter_challenge_button = Button::new(
             TeamView::OpenToChallenge.to_string(),
@@ -143,7 +143,7 @@ impl TeamListPanel {
         )
         .bold()
         .set_hotkey(ui_key::CYCLE_VIEW)
-        .set_hover_text("View all teams that can be currently challenged to a game.");
+        .set_hover_text("View all crews that can be currently challenged to a game.");
 
         let mut filter_peers_button = Button::new(
             TeamView::Peers.to_string(),
@@ -153,7 +153,7 @@ impl TeamListPanel {
         ).bold()
         .set_hotkey(ui_key::CYCLE_VIEW)
         .set_hover_text(
-            "View all teams received from the network (i.e. teams controlled by other players online)."
+            "View all crews received from the network (i.e. crews controlled by other players online)."
                 ,
         );
         match self.view {
@@ -169,7 +169,7 @@ impl TeamListPanel {
         if !self.team_ids.is_empty() {
             let mut options = vec![];
             for team_id in self.team_ids.iter() {
-                let team = if let Some(team) = world.get_team(team_id) {
+                let team = if let Some(team) = world.teams.get(team_id) {
                     team
                 } else {
                     continue;
@@ -190,12 +190,12 @@ impl TeamListPanel {
             let list = selectable_list(options);
 
             frame.render_stateful_interactive_widget(
-                list.block(default_block().title("Teams ↓/↑")),
+                list.block(default_block().title("Crews ↓/↑")),
                 split[3],
                 &mut ClickableListState::default().with_selected(self.index),
             );
         } else {
-            frame.render_widget(default_block().title("Teams"), split[3]);
+            frame.render_widget(default_block().title("Crews"), split[3]);
         }
     }
 
@@ -212,7 +212,7 @@ impl TeamListPanel {
             return Ok(());
         };
 
-        let team = world.get_team_or_err(&self.team_ids[index])?;
+        let team = world.teams.get_or_err(&self.team_ids[index])?;
         self.current_team_players_length = team.player_ids.len();
         let vertical_split = Layout::vertical([
             Constraint::Length(PLAYER_IMAGE_HEIGHT as u16 / 2), //players
@@ -273,7 +273,7 @@ impl TeamListPanel {
 
             frame.render_interactive_widget(button, button_area);
 
-            let player = world.get_player_or_err(&team.player_ids[i])?;
+            let player = world.players.get_or_err(&team.player_ids[i])?;
 
             if let Ok(lines) = self.gif_map.player_frame_lines(player, self.tick) {
                 frame.render_widget(Paragraph::new(lines).centered(), player_img_split[i + 1]);
@@ -339,7 +339,7 @@ impl TeamListPanel {
                 .skip(MIN_PLAYERS_PER_GAME)
                 .enumerate()
             {
-                if let Some(player) = world.get_player(player_id) {
+                if let Some(player) = world.players.get(player_id) {
                     let info = format!("{}\n", player.info.short_name());
                     let skills = player.current_skill_array();
                     let best_role = GamePosition::best(skills);
@@ -399,7 +399,7 @@ impl TeamListPanel {
             world,
             world.team_rating(&team.id).unwrap_or_default(),
             false,
-            world.get_team(&team.id).is_some(),
+            world.teams.contains_key(&team.id),
             &mut self.gif_map,
             self.tick,
             frame,
@@ -458,7 +458,7 @@ impl Screen for TeamListPanel {
                 .all_team_ids
                 .iter()
                 .filter(|&team_id| {
-                    let team = world.get_team_or_err(team_id).unwrap();
+                    let team = world.teams.get_or_err(team_id).unwrap();
                     self.view.rule(team, own_team)
                 })
                 .copied()
@@ -476,7 +476,8 @@ impl Screen for TeamListPanel {
             if index < self.team_ids.len() {
                 self.selected_team_id = self.team_ids[index];
                 let players = world
-                    .get_team_or_err(&self.selected_team_id)?
+                    .teams
+                    .get_or_err(&self.selected_team_id)?
                     .player_ids
                     .clone();
                 if self.player_index < players.len() {
@@ -506,7 +507,7 @@ impl Screen for TeamListPanel {
         self.build_left_panel(frame, world, left_right_split[0]);
         if self.all_team_ids.is_empty() {
             frame.render_widget(
-                Paragraph::new(" No teams yet!"),
+                Paragraph::new(" No crews yet!"),
                 left_right_split[1].inner(Margin::new(1, 1)),
             );
             return Ok(());
