@@ -12,9 +12,10 @@ use super::{
 use crate::app_version;
 use crate::core::{AsteroidUpgradeTarget, UpgradeableElement};
 use crate::game_engine::game::Game;
-use crate::game_engine::Tournament;
+use crate::game_engine::{Tournament, TournamentId, TournamentType};
+use crate::network::types::TournamentRequestState;
 use crate::network::{challenge::Challenge, trade::Trade};
-use crate::types::PlayerMap;
+use crate::types::{HashMapWithResult, PlayerMap};
 use crate::ui::tournament_panel::TournamentView;
 use crate::ui::ui_key;
 use crate::{
@@ -113,6 +114,7 @@ pub enum UiCallback {
     GoToTrade {
         trade: Trade,
     },
+
     SetTeamColors {
         color: ColorPreset,
         channel: usize,
@@ -121,8 +123,6 @@ pub enum UiCallback {
         tactic: Tactic,
     },
     SetNextTeamTactic,
-    NextUiTab,
-    PreviousUiTab,
     SetUiTab {
         ui_tab: UiTab,
     },
@@ -178,9 +178,26 @@ pub enum UiCallback {
     },
 
     OrganizeNewTournament {
-        max_participants: usize,
-        starting_at: Tick,
+        tournament_type: TournamentType,
     },
+
+    CancelTournament {
+        tournament_id: TournamentId,
+        error_message: String,
+    },
+
+    ConfirmTournamentParticipants {
+        tournament_id: TournamentId,
+    },
+
+    SendInitializedTournament {
+        tournament_id: TournamentId,
+    },
+
+    RegisterToTournament {
+        tournament_id: TournamentId,
+    },
+
     GeneratePlayerTeam {
         name: String,
         home_planet: PlanetId,
@@ -323,7 +340,8 @@ impl UiCallback {
             app.ui.team_panel.update(&app.world)?;
             let team_id = app
                 .world
-                .get_player_or_err(&player_id)?
+                .players
+                .get_or_err(&player_id)?
                 .team
                 .ok_or(anyhow!("Player {player_id:?} has no team"))?;
 
@@ -339,7 +357,8 @@ impl UiCallback {
                 app.ui.team_panel.set_index(index);
                 let player_index = app
                     .world
-                    .get_team_or_err(&team_id)?
+                    .teams
+                    .get_or_err(&team_id)?
                     .player_ids
                     .iter()
                     .position(|&x| x == player_id)
@@ -392,7 +411,7 @@ impl UiCallback {
 
     fn go_to_home_planet(team_id: TeamId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let team = app.world.get_team_or_err(&team_id)?;
+            let team = app.world.teams.get_or_err(&team_id)?;
             app.ui
                 .galaxy_panel
                 .go_to_planet(team.home_planet_id, ZoomLevel::In);
@@ -404,11 +423,11 @@ impl UiCallback {
 
     fn go_to_current_team_planet(team_id: TeamId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let team = app.world.get_team_or_err(&team_id)?;
+            let team = app.world.teams.get_or_err(&team_id)?;
             let target = match team.current_location {
                 TeamLocation::OnPlanet {
                     planet_id: current_planet_id,
-                } => app.world.get_planet_or_err(&current_planet_id)?,
+                } => app.world.planets.get_or_err(&current_planet_id)?,
                 TeamLocation::Travelling { .. } => {
                     return Err(anyhow!("Team is travelling"));
                 }
@@ -429,13 +448,13 @@ impl UiCallback {
 
     fn go_to_current_player_planet(player_id: PlayerId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let player = app.world.get_player_or_err(&player_id)?;
+            let player = app.world.players.get_or_err(&player_id)?;
 
             match player.current_location {
                 PlayerLocation::OnPlanet {
                     planet_id: current_planet_id,
                 } => {
-                    let target = app.world.get_planet_or_err(&current_planet_id)?;
+                    let target = app.world.planets.get_or_err(&current_planet_id)?;
                     app.ui.galaxy_panel.go_to_planet(target.id, ZoomLevel::In);
                     app.ui.switch_to(super::ui_screen::UiTab::Galaxy);
                 }
@@ -495,7 +514,7 @@ impl UiCallback {
     fn challenge_team(team_id: TeamId) -> AppCallback {
         Box::new(move |app: &mut App| {
             let own_team = app.world.get_own_team()?;
-            let team = app.world.get_team_or_err(&team_id)?;
+            let team = app.world.teams.get_or_err(&team_id)?;
 
             // Challenge to network team.
             if let Some(peer_id) = team.peer_id {
@@ -551,14 +570,14 @@ impl UiCallback {
         Box::new(move |app: &mut App| {
             let own_team = app.world.get_own_team()?;
 
-            let target_player = app.world.get_player_or_err(&target_player_id)?;
+            let target_player = app.world.players.get_or_err(&target_player_id)?;
             let target_team = if let Some(team_id) = target_player.team {
-                app.world.get_team_or_err(&team_id)?
+                app.world.teams.get_or_err(&team_id)?
             } else {
                 return Err(anyhow!("Target player has no team"));
             };
 
-            let proposer_player = app.world.get_player_or_err(&proposer_player_id)?;
+            let proposer_player = app.world.players.get_or_err(&proposer_player_id)?;
             own_team.can_trade_players(proposer_player, target_player, target_team)?;
 
             // Network trade
@@ -591,19 +610,6 @@ impl UiCallback {
             Ok(Some("Trade Rejected".to_string()))
         })
     }
-    fn next_ui_tab() -> AppCallback {
-        Box::new(move |app: &mut App| {
-            app.ui.next_tab();
-            Ok(None)
-        })
-    }
-
-    fn previous_ui_tab() -> AppCallback {
-        Box::new(move |app: &mut App| {
-            app.ui.previous_tab();
-            Ok(None)
-        })
-    }
 
     fn set_ui_tab(ui_tab: UiTab) -> AppCallback {
         Box::new(move |app: &mut App| {
@@ -656,13 +662,14 @@ impl UiCallback {
         })
     }
 
-    fn organize_new_tournament(max_participants: usize, starting_at: Tick) -> AppCallback {
+    fn organize_new_tournament(tournament_type: TournamentType) -> AppCallback {
         Box::new(move |app: &mut App| {
             let own_team = app
                 .world
                 .teams
                 .get_mut(&app.world.own_team_id)
                 .expect("Own team should exists");
+
             own_team.can_organize_tournament()?;
 
             let planet_id = own_team
@@ -674,35 +681,20 @@ impl UiCallback {
                 .get(&planet_id)
                 .expect("Space cove planet should exist.");
 
-            let mut tournament =
-                Tournament::new(own_team, max_participants, starting_at)?.on_planet(planet);
+            let tournament = Tournament::new(own_team, tournament_type)?.on_planet(planet);
 
-            let mut players = PlayerMap::new();
-            for &player_id in own_team.player_ids.iter().take(MAX_PLAYERS_PER_GAME) {
-                let player = if let Some(player) = app.world.players.get(&player_id) {
-                    player.clone()
-                } else {
-                    return Err(anyhow!("Could not find player {player_id}"));
-                };
-                players.insert(player_id, player);
-            }
+            own_team.is_organizing_tournament = Some(tournament.id);
 
-            tournament.register_team(own_team, players)?;
             app.world
                 .tournaments
                 .insert(tournament.id, tournament.clone());
 
-            own_team.tournament_registration_state = TournamentRegistrationState::Confirmed {
-                tournament_id: tournament.id,
-            };
-
             app.ui.push_popup_to_top(PopupMessage::Ok {
                 message: format!(
-                    "New tournament created! \nStarting on {} {} on {} \n {}/{} registered teams",
-                    starting_at.formatted_as_date(),
-                    starting_at.formatted_as_time(),
+                    "New tournament created! \nRegistrations closing on {} {} at {} \n {} max participants.",
+                    tournament.registrations_closing_at.formatted_as_date(),
+                    tournament.registrations_closing_at.formatted_as_time(),
                     planet.name,
-                    tournament.participants.len(),
                     tournament.max_participants
                 ),
                 is_skippable: true,
@@ -764,7 +756,7 @@ impl UiCallback {
 
     fn next_training_focus(team_id: TeamId) -> AppCallback {
         Box::new(move |app: &mut App| {
-            let mut team = app.world.get_team_or_err(&team_id)?.clone();
+            let mut team = app.world.teams.get_or_err(&team_id)?.clone();
             team.can_change_training_focus()?;
 
             let new_focus = match team.training_focus {
@@ -782,7 +774,7 @@ impl UiCallback {
     fn travel_to_planet(planet_id: PlanetId) -> AppCallback {
         Box::new(move |app: &mut App| {
             let mut own_team = app.world.get_own_team()?.clone();
-            let target_planet = app.world.get_planet_or_err(&planet_id)?;
+            let target_planet = app.world.planets.get_or_err(&planet_id)?;
 
             let mut current_planet = match own_team.current_location {
                 TeamLocation::OnPlanet {
@@ -791,7 +783,7 @@ impl UiCallback {
                     if current_planet_id == planet_id {
                         return Err(anyhow!("Already on planet"));
                     }
-                    app.world.get_planet_or_err(&current_planet_id)?.clone()
+                    app.world.planets.get_or_err(&current_planet_id)?.clone()
                 }
                 TeamLocation::Travelling { .. } => return Err(anyhow!("Team is travelling")),
                 TeamLocation::Exploring { .. } => return Err(anyhow!("Team is exploring")),
@@ -849,7 +841,7 @@ impl UiCallback {
             };
 
             for player in own_team.player_ids.iter() {
-                let mut player = app.world.get_player_or_err(player)?.clone();
+                let mut player = app.world.players.get_or_err(player)?.clone();
                 player.set_jersey(&pirate_jersey);
                 app.world.players.insert(player.id, player);
             }
@@ -876,7 +868,7 @@ impl UiCallback {
                 }
             };
 
-            let mut around_planet = app.world.get_planet_or_err(&planet_id)?.clone();
+            let mut around_planet = app.world.planets.get_or_err(&planet_id)?.clone();
             own_team.can_explore_around_planet(&around_planet, duration)?;
 
             own_team.current_location = TeamLocation::Exploring {
@@ -902,7 +894,7 @@ impl UiCallback {
             };
 
             for player_id in own_team.player_ids.iter() {
-                let mut player = app.world.get_player_or_err(player_id)?.clone();
+                let mut player = app.world.players.get_or_err(player_id)?.clone();
                 player.set_jersey(&pirate_jersey);
                 app.world.players.insert(player.id, player);
             }
@@ -952,7 +944,7 @@ impl UiCallback {
                         filename.clone(),
                         planet_id,
                     )?;
-                    let mut current_planet = app.world.get_planet_or_err(&planet_id)?.clone();
+                    let mut current_planet = app.world.planets.get_or_err(&planet_id)?.clone();
                     current_planet.team_ids.retain(|&x| x != own_team.id);
                     app.world.planets.insert(current_planet.id, current_planet);
 
@@ -960,7 +952,7 @@ impl UiCallback {
                         planet_id: asteroid_id,
                     };
 
-                    let mut asteroid = app.world.get_planet_or_err(&asteroid_id)?.clone();
+                    let mut asteroid = app.world.planets.get_or_err(&asteroid_id)?.clone();
                     asteroid.team_ids.push(own_team.id);
                     asteroid.version += 1;
 
@@ -1075,9 +1067,7 @@ impl UiCallback {
 
             // Special handling for space cove
             if upgrade.target == AsteroidUpgradeTarget::SpaceCove {
-                own_team.space_cove = SpaceCoveState::Pending {
-                    planet_id: asteroid_id,
-                }
+                own_team.space_cove = Some(SpaceCove::under_construction(asteroid_id));
             }
 
             asteroid.pending_upgrade = Some(upgrade);
@@ -1110,14 +1100,37 @@ impl UiCallback {
                 let own_team = if let Some(team) = app.world.teams.get_mut(&app.world.own_team_id) {
                     team
                 } else {
-                    app.ui.push_popup(PopupMessage::Error {
-                        message: format!("Cannot find own team {}", app.world.own_team_id),
-                        tick: Tick::now(),
-                    });
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!("Cannot find own team {}", app.world.own_team_id),
+                        log::Level::Error,
+                    );
                     return Ok(None);
                 };
-                own_team.space_cove = SpaceCoveState::Ready {
-                    planet_id: asteroid_id,
+                if let Some(cove) = own_team.space_cove.as_mut() {
+                    if cove.planet_id != asteroid_id {
+                        app.ui.push_log_event(
+                            Tick::now(),
+                            None,
+                            format!("Cannot finalize space cove upgrade: mismatching planet id."),
+                            log::Level::Error,
+                        );
+                        return Ok(None);
+                    }
+
+                    cove.finish_contruction();
+                    own_team.version += 1;
+                } else {
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!(
+                            "Cannot finalize space cove upgrade: no space cove under construction.",
+                        ),
+                        log::Level::Error,
+                    );
+                    return Ok(None);
                 }
             }
 
@@ -1273,8 +1286,6 @@ impl UiCallback {
                 Ok(None)
             }
             Self::GoToTrade { trade } => Self::go_to_trade(trade.clone())(app),
-            Self::NextUiTab => Self::next_ui_tab()(app),
-            Self::PreviousUiTab => Self::previous_ui_tab()(app),
             Self::SetUiTab { ui_tab } => Self::set_ui_tab(*ui_tab)(app),
             Self::NextPanelIndex => Self::next_panel_index()(app),
             Self::PreviousPanelIndex => Self::previous_panel_index()(app),
@@ -1350,7 +1361,7 @@ impl UiCallback {
 
                 // Remove space cove if it was on the abandoned asteroid
                 if matches!(own_team.has_space_cove_on(), Some(id) if id == *asteroid_id) {
-                    own_team.space_cove = SpaceCoveState::None;
+                    own_team.space_cove = None;
                 }
                 app.ui.close_popup();
                 Ok(None)
@@ -1391,7 +1402,7 @@ impl UiCallback {
             }
 
             Self::Drink { player_id } => {
-                let mut player = app.world.get_player_or_err(player_id)?.clone();
+                let mut player = app.world.players.get_or_err(player_id)?.clone();
                 player.can_drink(&app.world)?;
 
                 let morale_bonus = if matches!(player.special_trait, Some(Trait::Spugna)) {
@@ -1411,7 +1422,8 @@ impl UiCallback {
 
                 let mut team = app
                     .world
-                    .get_team_or_err(&player.team.expect("Player should have team"))?
+                    .teams
+                    .get_or_err(&player.team.expect("Player should have team"))?
                     .clone();
 
                 team.sub_resource(Resource::RUM, 1)?;
@@ -1455,7 +1467,7 @@ impl UiCallback {
                             .map(|(&id, _)| id),
                     };
                     if let Some(to) = portal_target_id {
-                        let portal_target = app.world.get_planet_or_err(&to)?;
+                        let portal_target = app.world.planets.get_or_err(&to)?;
                         // We set the new target to the portal_target
                         let from = match team.current_location {
                             TeamLocation::OnPlanet { .. }
@@ -1507,10 +1519,180 @@ impl UiCallback {
                 players.clone(),
                 spaceship.clone(),
             )(app),
-            Self::OrganizeNewTournament {
-                max_participants,
-                starting_at,
-            } => Self::organize_new_tournament(*max_participants, *starting_at)(app),
+
+            Self::OrganizeNewTournament { tournament_type } => {
+                Self::organize_new_tournament(*tournament_type)(app)
+            }
+
+            Self::CancelTournament {
+                tournament_id,
+                error_message,
+            } => {
+                if let Some(tournament) = app.world.tournaments.get_mut(tournament_id) {
+                    tournament.cancel();
+
+                    if app.world.own_team_id == tournament.organizer_id {
+                        for &team_id in tournament.registered_teams.keys() {
+                            if team_id == tournament.organizer_id {
+                                continue;
+                            }
+                            let _ = app.network_handler.send_tournament_registration_request(
+                                *tournament_id,
+                                team_id,
+                                None,
+                                TournamentRequestState::Cancellation {
+                                    reason: error_message.clone(),
+                                },
+                            );
+                        }
+
+                        let own_team = app.world.teams.get_mut_or_err(&app.world.own_team_id)?;
+                        own_team.is_organizing_tournament = None;
+                    }
+
+                    if tournament.is_team_registered(&app.world.own_team_id) {
+                        let own_team = app.world.teams.get_mut_or_err(&app.world.own_team_id)?;
+                        own_team.tournament_registration_state = TournamentRegistrationState::None;
+                        app.ui.push_popup(PopupMessage::Ok {
+                            message: format!("{} tournament got cancelled.", tournament.name()),
+                            tick: Tick::now(),
+                            is_skippable: false,
+                        });
+                    }
+
+                    app.ui.push_log_event(
+                        Tick::now(),
+                        None,
+                        format!(
+                            "{} tournament got cancelled: {}",
+                            tournament.name(),
+                            error_message
+                        ),
+                        log::Level::Warn,
+                    );
+                }
+
+                Ok(None)
+            }
+
+            Self::ConfirmTournamentParticipants { tournament_id } => {
+                if let Some(tournament) = app.world.tournaments.get_mut(tournament_id) {
+                    // Check that team is organizer and confirm it automatically.
+                    if tournament.organizer_id != app.world.own_team_id {
+                        return Err(anyhow!("Own team is not tournament organizer."));
+                    }
+
+                    let own_team = app
+                        .world
+                        .teams
+                        .get_mut(&app.world.own_team_id)
+                        .expect("Own team should exist in world.");
+                    let own_peer_id = *app.network_handler.own_peer_id();
+                    let mut team = own_team.clone();
+
+                    // FIXME: All these dances with the peerId can be avoided if we set it for the own team when loading, now that we can.
+                    // Set peer id, so everyone gets it right when tournament is sent around.
+                    team.peer_id = Some(own_peer_id);
+
+                    let mut players = PlayerMap::new();
+                    for &player_id in own_team.player_ids.iter().take(MAX_PLAYERS_PER_GAME) {
+                        let player = if let Some(player) = app.world.players.get(&player_id) {
+                            let mut p = player.clone();
+                            p.peer_id = Some(own_peer_id);
+                            p
+                        } else {
+                            return Err(anyhow!("Could not find player {player_id}"));
+                        };
+                        players.insert(player_id, player);
+                    }
+
+                    if tournament
+                        .confirm_team_registration(&team, players, Tick::now())
+                        .is_ok()
+                    {
+                        own_team.tournament_registration_state =
+                            TournamentRegistrationState::Confirmed {
+                                tournament_id: tournament.id,
+                            };
+                    }
+
+                    for &team_id in tournament.registered_teams.keys() {
+                        if team_id == app.world.own_team_id {
+                            continue;
+                        }
+
+                        let _ = app.network_handler.send_tournament_registration_request(
+                            *tournament_id,
+                            team_id,
+                            None,
+                            TournamentRequestState::ConfirmationRequest,
+                        );
+                    }
+                } else {
+                    log::warn!(
+                        "Invalid tournament id {tournament_id} for ConfirmTournamentParticipants."
+                    );
+                }
+
+                Ok(None)
+            }
+
+            Self::SendInitializedTournament { tournament_id } => {
+                if let Some(tournament) = app.world.tournaments.get(tournament_id) {
+                    let _ = app.network_handler.send_tournament(tournament.clone());
+                } else {
+                    log::warn!(
+                        "Invalid tournament id {tournament_id} for SendInitializedTournament."
+                    );
+                }
+
+                Ok(None)
+            }
+
+            Self::RegisterToTournament { tournament_id } => {
+                let mut team = app.world.get_own_team()?.clone();
+                team.peer_id = Some(*app.network_handler.own_peer_id());
+
+                if let Some(tournament) = app.world.tournaments.get(tournament_id) {
+                    team.can_register_to_tournament(tournament, Tick::now())?;
+                } else {
+                    log::warn!("Invalid tournament id {tournament_id} for RegisterToTournament.");
+                    return Ok(None);
+                };
+
+                let players = World::get_game_players_by_team(&app.world.players, &team)?;
+
+                let tournament = app
+                    .world
+                    .tournaments
+                    .get_mut(tournament_id)
+                    .expect("Tournament should exist here.");
+                // Organizer gets registered immediately.
+                if tournament.organizer_id == app.world.own_team_id {
+                    tournament.register_team(&team, players, Tick::now())?;
+                    let own_team = app.world.get_own_team_mut()?;
+                    own_team.tournament_registration_state =
+                        TournamentRegistrationState::Registered {
+                            tournament_id: *tournament_id,
+                        };
+                } else {
+                    let team_data = Some((team, players));
+                    app.network_handler.send_tournament_registration_request(
+                        *tournament_id,
+                        app.world.own_team_id,
+                        team_data,
+                        TournamentRequestState::RegistrationRequest,
+                    )?;
+
+                    let own_team = app.world.get_own_team_mut()?;
+                    own_team.tournament_registration_state = TournamentRegistrationState::Pending {
+                        tournament_id: *tournament_id,
+                    };
+                }
+
+                Ok(None)
+            }
+
             Self::CancelGeneratePlayerTeam => Self::cancel_generate_own_team()(app),
             Self::AssignBestTeamPositions => Self::assign_best_team_positions()(app),
             Self::SwapPlayerPositions {
@@ -1556,7 +1738,7 @@ impl UiCallback {
 
                 let should_spawn_asteroid = match own_team.current_location {
                     TeamLocation::OnPlanet { planet_id } => {
-                        let current_planet = app.world.get_planet_or_err(&planet_id)?;
+                        let current_planet = app.world.planets.get_or_err(&planet_id)?;
                         current_planet.asteroid_probability > 0.0
                             && own_team.asteroid_ids.len() < MAX_NUM_ASTEROID_PER_TEAM
                     }
@@ -1570,7 +1752,7 @@ impl UiCallback {
 
                 let gold_fragment_probability = match own_team.current_location {
                     TeamLocation::OnPlanet { planet_id } => {
-                        let current_planet = app.world.get_planet_or_err(&planet_id)?;
+                        let current_planet = app.world.planets.get_or_err(&planet_id)?;
                         0.001
                             + 0.075 * (current_planet.resources.value(&Resource::GOLD) as f64)
                                 / MAX_SKILL as f64
