@@ -15,9 +15,12 @@ use super::{
 };
 use crate::core::skill::{Skill, MAX_SKILL, MIN_SKILL};
 use crate::core::types::TeamBonus;
-use crate::core::{ChargeUnit, Honour, Shield, Shooter, SpaceshipComponent, UpgradeableElement};
-use crate::ui::ui_key;
+use crate::core::{
+    AsteroidUpgradeTarget, ChargeUnit, Honour, Planet, Shield, Shooter, SpaceshipComponent,
+    Upgrade, UpgradeableElement,
+};
 use crate::ui::utils::format_au;
+use crate::ui::{ui_key, PopupMessage};
 use crate::{
     core::{
         constants::*,
@@ -84,7 +87,7 @@ pub fn selectable_list<'a>(options: Vec<(String, Style)>) -> ClickableList<'a> {
 }
 
 pub fn go_to_planet_button<'a>(world: &World, planet_id: PlanetId) -> AppResult<Button<'a>> {
-    let planet_name = world.get_planet_or_err(&planet_id)?.name.as_str();
+    let planet_name = world.planets.get_or_err(&planet_id)?.name.as_str();
     Ok(Button::new(
         format!("Go to planet: {planet_name}"),
         UiCallback::GoToPlanet { planet_id },
@@ -101,7 +104,7 @@ pub fn go_to_space_cove_button<'a>() -> AppResult<Button<'a>> {
 
 pub fn teleport_button<'a>(world: &World, planet_id: PlanetId) -> AppResult<Button<'a>> {
     let own_team = world.get_own_team()?;
-    let planet = world.get_planet_or_err(&planet_id)?;
+    let planet = world.planets.get_or_err(&planet_id)?;
 
     let mut teleport_button = Button::new(
         format!("Teleport (-{} Rum)", own_team.player_ids.len()),
@@ -126,8 +129,8 @@ pub fn teleport_button<'a>(world: &World, planet_id: PlanetId) -> AppResult<Butt
 }
 
 pub fn go_to_team_home_planet_button<'a>(world: &World, team_id: &TeamId) -> AppResult<Button<'a>> {
-    let team = world.get_team_or_err(team_id)?;
-    let planet_name = &world.get_planet_or_err(&team.home_planet_id)?.name;
+    let team = world.teams.get_or_err(team_id)?;
+    let planet_name = &world.planets.get_or_err(&team.home_planet_id)?.name;
     Ok(Button::new(
         format!("Home planet {planet_name}"),
         UiCallback::GoToHomePlanet { team_id: team.id },
@@ -140,15 +143,15 @@ pub fn go_to_team_current_planet_button<'a>(
     world: &World,
     team_id: &TeamId,
 ) -> AppResult<Button<'a>> {
-    let team = world.get_team_or_err(team_id)?;
+    let team = world.teams.get_or_err(team_id)?;
     let go_to_team_current_planet_button = match team.current_location {
         TeamLocation::OnPlanet { planet_id } => Button::new(
-            format!("On planet {}", world.get_planet_or_err(&planet_id)?.name),
+            format!("On planet {}", world.planets.get_or_err(&planet_id)?.name),
             UiCallback::GoToCurrentTeamPlanet { team_id: team.id },
         )
         .set_hover_text(format!(
             "Go to planet {}",
-            world.get_planet_or_err(&planet_id)?.name
+            world.planets.get_or_err(&planet_id)?.name
         ))
         .set_hotkey(ui_key::ON_PLANET),
 
@@ -159,7 +162,7 @@ pub fn go_to_team_current_planet_button<'a>(
             duration,
             ..
         } => {
-            let to = world.get_planet_or_err(&to)?.name.to_string();
+            let to = world.planets.get_or_err(&to)?.name.to_string();
             let text = if started + duration > world.last_tick_short_interval + 3 * SECONDS {
                 format!("Travelling to {to}")
             } else {
@@ -173,7 +176,7 @@ pub fn go_to_team_current_planet_button<'a>(
             started,
             duration,
         } => {
-            let around_planet = world.get_planet_or_err(&around)?.name.to_string();
+            let around_planet = world.planets.get_or_err(&around)?.name.to_string();
             let text = if started + duration > world.last_tick_short_interval + 3 * SECONDS {
                 format!("Around {around_planet}")
             } else {
@@ -196,7 +199,7 @@ pub fn go_to_team_current_planet_button<'a>(
 }
 
 pub fn drink_button<'a>(world: &World, player_id: &PlayerId) -> AppResult<Button<'a>> {
-    let player = world.get_player_or_err(player_id)?;
+    let player = world.players.get_or_err(player_id)?;
     let can_drink = player.can_drink(world);
 
     let mut button = Button::new(
@@ -286,7 +289,7 @@ pub fn render_challenge_button(
 
     let challenge_button = if let Some(game_id) = team.current_game {
         // FIXME: The game is not necessarily part of the world if it's a network game.
-        let game_text = if let Ok(game) = world.get_game_or_err(&game_id) {
+        let game_text = if let Ok(game) = world.games.get_or_err(&game_id) {
             if let Some(action) = game.action_results.last() {
                 format!(
                     "{} {:>3}-{:<3} {}",
@@ -310,7 +313,7 @@ pub fn render_challenge_button(
         )
         .set_hover_text("Go to team's game")
         .set_hotkey(ui_key::GO_TO_GAME);
-        if world.get_game_or_err(&game_id).is_err() {
+        if world.games.get_or_err(&game_id).is_err() {
             b.disable(Some("Game is not visible"));
         }
         b
@@ -394,7 +397,7 @@ pub fn explore_button<'a>(world: &World, team: &Team) -> AppResult<Button<'a>> {
 
     match team.current_location {
         TeamLocation::OnPlanet { planet_id } => {
-            let planet = world.get_planet_or_err(&planet_id)?;
+            let planet = world.planets.get_or_err(&planet_id)?;
             let needed_fuel = (duration as f32 * team.spaceship_fuel_consumption_per_tick()) as u32;
             button = button.set_hover_text(
                 format!(
@@ -415,7 +418,7 @@ pub fn explore_button<'a>(world: &World, team: &Team) -> AppResult<Button<'a>> {
                 "Explore the space on autopilot. Hope to find resources, free pirates or more..."
                     .to_string(),
             );
-            let to = world.get_planet_or_err(&to)?.name.to_string();
+            let to = world.planets.get_or_err(&to)?.name.to_string();
             button.disable(Some(format!("Travelling to planet {to}")));
         }
         TeamLocation::Exploring { around, .. } => {
@@ -423,7 +426,7 @@ pub fn explore_button<'a>(world: &World, team: &Team) -> AppResult<Button<'a>> {
                 "Explore the space on autopilot. Hope to find resources, free pirates or more..."
                     .to_string(),
             );
-            let around_planet = world.get_planet_or_err(&around)?.name.to_string();
+            let around_planet = world.planets.get_or_err(&around)?.name.to_string();
             button.disable(Some(format!("Exploring around planet {around_planet}")));
         }
         TeamLocation::OnSpaceAdventure { .. } => {
@@ -440,7 +443,7 @@ pub fn space_adventure_button<'a>(world: &World, team: &Team) -> AppResult<Butto
 
     match team.current_location {
         TeamLocation::OnPlanet { planet_id } => {
-            let planet = world.get_planet_or_err(&planet_id)?;
+            let planet = world.planets.get_or_err(&planet_id)?;
             button = button.set_hover_text(format!(
                 "Start a space adventure around {} to collect resources and more...",
                 planet.name,
@@ -456,14 +459,14 @@ pub fn space_adventure_button<'a>(world: &World, team: &Team) -> AppResult<Butto
             button = button.set_hover_text(
                 "Start a space adventure to manually collect resources and more...".to_string(),
             );
-            let to = world.get_planet_or_err(&to)?.name.to_string();
+            let to = world.planets.get_or_err(&to)?.name.to_string();
             button.disable(Some(format!("Travelling to planet {to}")));
         }
         TeamLocation::Exploring { around, .. } => {
             button = button.set_hover_text(
                 "Start a space adventure to manually collect resources and more...".to_string(),
             );
-            let around_planet = world.get_planet_or_err(&around)?.name.to_string();
+            let around_planet = world.planets.get_or_err(&around)?.name.to_string();
             button.disable(Some(format!("Exploring around planet {around_planet}")));
         }
         TeamLocation::OnSpaceAdventure { .. } => {
@@ -928,6 +931,154 @@ fn honour_lines<'a>(team_honours: &HashSet<Honour>) -> Vec<HoverTextLine<'a>> {
     ]
 }
 
+// FIXME: unify the following two functions.
+// It's unclear how to get custom description at the moment for the spaceship upgrades, maybe delegate it to the upgrade target?
+pub fn render_available_upgrades<U: UpgradeableElement>(
+    pending_upgrade: Option<Upgrade<U>>,
+    possible_upgrade: Option<Upgrade<U>>,
+    world: &World,
+    own_team: &Team,
+    frame: &mut UiFrame,
+    area: Rect,
+) -> AppResult<()> {
+    if let Some(upgrade) = pending_upgrade {
+        let countdown = (upgrade.started + upgrade.duration)
+            .saturating_sub(world.last_tick_short_interval)
+            .formatted();
+
+        frame.render_widget(
+            Paragraph::new(vec![
+                Line::from(Span::styled(
+                    format!("Building {}", upgrade.target),
+                    UiStyle::HEADER.bold(),
+                )),
+                Line::from(countdown),
+            ])
+            .centered(),
+            area,
+        );
+    } else if let Some(upgrade) = possible_upgrade {
+        {
+            let mut lines = vec![Line::from(Span::styled(
+                format!("{} upgrade cost", upgrade.target),
+                UiStyle::HEADER.bold(),
+            ))];
+            lines.append(&mut upgrade_resources_lines(upgrade.target, own_team));
+            frame.render_widget(Paragraph::new(lines).centered(), area);
+        }
+    } else {
+        frame.render_widget(Paragraph::new("Nothing left to build").centered(), area);
+    }
+
+    Ok(())
+}
+
+pub fn render_available_spaceship_upgrades(
+    pending_upgrade: Option<Upgrade<SpaceshipUpgradeTarget>>,
+    possible_upgrade: Option<Upgrade<SpaceshipUpgradeTarget>>,
+    world: &World,
+    own_team: &Team,
+    frame: &mut UiFrame,
+    area: Rect,
+) -> AppResult<()> {
+    if let Some(upgrade) = pending_upgrade {
+        let header = match upgrade.target {
+            SpaceshipUpgradeTarget::Repairs { .. } => "Repairing spaceship".to_string(),
+            target => format!("Upgrading {target}"),
+        };
+
+        let countdown = (upgrade.started + upgrade.duration)
+            .saturating_sub(world.last_tick_short_interval)
+            .formatted();
+
+        frame.render_widget(
+            Paragraph::new(vec![
+                Line::from(vec![Span::styled(header, UiStyle::HEADER.bold())]),
+                Line::from(countdown),
+            ])
+            .centered(),
+            area,
+        )
+    } else if let Some(upgrade) = possible_upgrade {
+        {
+            let mut lines =
+                vec![Line::from(Span::styled("Upgrade cost", UiStyle::HEADER.bold())).centered()];
+            lines.append(&mut upgrade_resources_lines(upgrade.target, own_team));
+            frame.render_widget(Paragraph::new(lines).centered(), area);
+        }
+    }
+
+    Ok(())
+}
+
+pub fn render_build_asteroid_upgrade_button(
+    asteroid: &Planet,
+    possible_upgrade: Option<Upgrade<AsteroidUpgradeTarget>>,
+    own_team: &Team,
+    frame: &mut UiFrame,
+    area: Rect,
+) -> AppResult<()> {
+    if let Some(pending_upgrade) = asteroid.pending_upgrade {
+        let build_button = Button::new(
+            format!("Building {}", pending_upgrade.target),
+            UiCallback::None,
+        )
+        .set_hover_text(format!(
+            "Building {} on {}",
+            pending_upgrade.target, asteroid.name
+        ))
+        .disabled(Some(format!("Already building {}", pending_upgrade.target)));
+
+        frame.render_interactive_widget(build_button, area);
+    } else if let Some(upgrade) = possible_upgrade {
+        let on_click = if upgrade.target == AsteroidUpgradeTarget::SpaceCove {
+            UiCallback::PushUiPopup {
+                popup_message: PopupMessage::BuildSpaceCove {
+                    asteroid_name: asteroid.name.clone(),
+                    asteroid_id: asteroid.id,
+                    tick: Tick::now(),
+                },
+            }
+        } else {
+            UiCallback::SetAsteroidPendingUpgrade {
+                asteroid_id: asteroid.id,
+                upgrade,
+            }
+        };
+
+        let mut build_button = Button::new(
+            format!(
+                "Build {} ({})",
+                upgrade.target,
+                upgrade.duration.formatted()
+            ),
+            on_click,
+        )
+        .set_hotkey(ui_key::BUILD_ASTEROID_UPGRADE)
+        .set_hover_text(upgrade.target.description());
+
+        if upgrade.target == AsteroidUpgradeTarget::SpaceCove {
+            build_button = build_button.block(default_block().border_style(UiStyle::WARNING));
+        }
+
+        let can_upgrade_asteroid = own_team.can_upgrade_asteroid(asteroid, &upgrade);
+        if let Err(e) = can_upgrade_asteroid.as_ref() {
+            build_button.disable(Some(e.to_string()));
+        }
+
+        frame.render_interactive_widget(build_button, area);
+    } else {
+        let build_button = Button::new(
+            format!("Nothing left to build on {}", asteroid.name),
+            UiCallback::None,
+        )
+        .disabled(None::<String>);
+        frame.render_interactive_widget(build_button, area);
+    }
+
+    Ok(())
+}
+
 pub fn render_spaceship_upgrade(
     team: &Team,
     upgrade_target: SpaceshipUpgradeTarget,
